      subroutine boost(q,pboost,qprime)
      implicit real*8 (a-h,o-z)

c this subroutine performs the boost according to the vector pboost of 
c  the fourvector q to the fourvector qprime. The vector q and the 
c   resulting qprime might also have the same name in the call       
c  Let's call rmboost the mass of pboost. With this transformation
c  if q= (rmboost,0,0,0), qprime=pboost 
    

      dimension q(0:3), pboost(0:3), qprime (0:3)

      rmboost=pboost(0)**2-pboost(1)**2-pboost(2)**2-pboost(3)**2
      rmboost=sqrt(max(rmboost,0.d0))
      aux=(q(0)*pboost(0)+q(1)*pboost(1)+q(2)*pboost(2)+q(3)*pboost(3))
     &      /rmboost
      aaux=(aux+q(0))/(pboost(0)+rmboost)
      qprime(0)=aux
      qprime(1)=q(1)+aaux*pboost(1)
      qprime(2)=q(2)+aaux*pboost(2)
      qprime(3)=q(3)+aaux*pboost(3)

      return
      end

      subroutine boostinv(q,pboost,qprime)
      implicit real*8 (a-h,o-z)

c this subroutine performs the inverse boost according to the vector 
c  pboost  ( which corresponds to the boost according to pboost(0), 
c  -pboost(i) ) of the fourvector q to the fourvector qprime. 
c  The vector q and the resulting qprime might also have the same 
c  name in the call       
c  Let's call rmboost the mass of pboost. With this transformation
c  if qprime= (rmboost,0,0,0), q=pboost  

      dimension q(0:3), pboost(0:3), qprime (0:3)

      rmboost=pboost(0)**2-pboost(1)**2-pboost(2)**2-pboost(3)**2
      rmboost=sqrt(max(rmboost,0.d0))
      aux=(q(0)*pboost(0)-q(1)*pboost(1)-q(2)*pboost(2)-q(3)*pboost(3))
     &      /rmboost
      aaux=(aux+q(0))/(pboost(0)+rmboost)
      qprime(0)=aux
      qprime(1)=q(1)-aaux*pboost(1)
      qprime(2)=q(2)-aaux*pboost(2)
      qprime(3)=q(3)-aaux*pboost(3)

      return
      end




      subroutine rot(p,q,pp,ierr)

* the subroutine performs a rotation such that the 3-vector p goes into
* the 3-vector pp. The rotation is such that the vector q', of the same
*  lenght as q but along the z axis, becomes q after the rotation
      
      implicit real*8 (a-h,o-z)

      dimension pp(3), p(3), q(3) 

      ierr=0

      qmod=sqrt(q(1)**2+q(2)**2+q(3)**2)
      if (qmod.le.0.d0) then
        ierr=1
        return
      endif

      cth=q(3)/qmod
      sth=1.d0-cth**2
      if (sth.lt.0.d0) then
        ierr=1
        return
      elseif (sth.eq.0.d0) then
        pp(1)=p(1)
        pp(2)=p(2)
        pp(3)=p(3)
        return
      endif
      sth=sqrt(sth)
      cfi=q(1)/qmod/sth
      sfi=q(2)/qmod/sth

c  avoid possible problems if p and pp are the same vector:
      
      p1=p(1)
      p2=p(2)
      p3=p(3)

c  perform the rotation

      pp(1)=cth*cfi*p1-sfi*p2+sth*cfi*p3
      pp(2)=cth*sfi*p1+cfi*p2+sth*sfi*p3
      pp(3)=-sth   *p1       +cth*    p3


      return
      end




