************************************************************************
*                                                                      *
* WPHACT: a Monte Carlo program for four-fermion final state processes *
*                          at e+ e- colliders                          *
*                                                                      *
*    Authors: Elena Accomando, Alessandro Ballestrero, Ezio Maina      *
*                                                                      *
*                           version 1.9                                *
*                                                                      *
************************************************************************
*
*  This version  contains several completely new features.
*  In particular, it is the first official version with completely
*  massive matrix elements. The user can now choose among fully massive
*   calculations, massless ones and processes in which only b's
*   (or taus, or c's) masses are exactly accounted for.
*  The last two choices were present in older versions of WPHACT.
*  They are much faster than the fully massive one and the difference
*   for most processes and cuts is of the order of the per mille.
*
*  The use of massive ME is however unavoidable for single W, single Z,
*   gamma gamma studies and in general for configurations with
*   very low separation angles and invariant masses.
*
*  For single W studies it is also possible to use the Im. fermion loop
*   calculations
*
************************************************************************
*
* This program,  its steering file, explanations and examples, as
* well as other versions of WPHACT can be found with a link to
*             http://www.to.infn.it/~ballestr/wphact
*                    and its subdirectories
*
* For questions, comments, remarks and requests, send a mail to
*                     ballestrero@to.infn.it
*
************************************************************************
************************************************************************




      IMPLICIT REAL*8 (a-b,d-h,o-z)
      IMPLICIT double COMPLEX (c)
      CHARACTER*60 string

      DIMENSION region(18),y1(50,10),y2(50,10),iv(32),iv1(32),iv2(32)
*el6
      DIMENSION iext(3:6)
*endel6
* em2
      COMMON/abpara/rmx1,gamx1,gx1,rmx2,gamx2,gx2,beta,rlim,s_col,
     &  x1_min,x2_min,smin,emcoupl,estrinf1,estrinf2,estrmed2,
     &  rmx3,gamx3,gx3,sepmass1low,sepmass1high,
     &  sepmass2low,sepmass2high
*el5 (iext)
      COMMON/abopzi/isr,ipr,ips,iccnc,iproc,ich,ichcj,ips_cc,ips_nc,
     &  icos,icch,isusy,iha,icut,igwcomp,igzcomp,ighcomp,iswgcomp,
     &  irmhcomp,imass,ismallangle,ism34,ism56,iext
*endel5
* endem2
      COMMON/abparb/rmw2,gamw,rmz2,rmt2,rcotw,pi,alfa_me,qcdcoupl,
     &            qcdcor_cc,qcdcor_nc,qcdcor_h
      COMMON/abparc/czipr,ccz,cwipr,ccw,chipr,cch,caipr,cca
      COMMON/abcoup/fer,fel,zer,zel,f3l,f4l,f5l,f6l,z3l,z4l,z5l,z6l,
     &  f3r,f4r,f5r,f6r,z3r,z4r,z5r,z6r,wcl,delz,xf,xz,yf,yz,zz
      COMMON/abcopl/zvl,zvr,fqdl,fqdr,zqdl,zqdr,fqul,fqur,zqul,zqur
      COMMON/abflag/icc,icc3,i3e,i4e,i3q,i5q,iqu,i34e,i56ve,ibbveve,iid,
     &  imix,icoul,istrcor,idownl,idownr,ianc
* el2
     &  ,izz,izg,izg34,izg56,inc08,igamgam,itch
* endel2
      COMMON/abhigg/rmb,rmb2,rmh,rmh2,gamh,rhzz,rhww,rhbb
      COMMON/absusy/rma,rma2,rzha,rabb
      COMMON/abcuts/e_min(3:6),e_max(3:6),thbeam_min(3:6),
     &  thbeam_max(3:6),thsep_min(6),thsep_max(6),rm_min(6),rm_max(6),
     &  beamcut_min(3:6),beamcut_max(3:6),sepcut_min(6),sepcut_max(6),
     &  rm_min2(6),rm_max2(6),pt_min(3:6),pt_max(3:6),e_cm
      COMMON/abinpu/rmw,rmz,rmb_run,rmc,rmc_run,rmtau,gamz,gf,s2w,
     &  alfainv,alfas_cc,alfas_nc,alfas_h,gauwidth
      COMMON /abrann/ idum
      COMMON/absalv/iv,iy,idum2
* sa2 added rmt
      COMMON/abmass/ rme,rmmu,rmu,rmd,rms,rmt,xm(6),xmp5,xmp
* sa2end
*
* sa1 nella successiva riga ho messo nintmax a 10 invece che 50
      PARAMETER (ndismax=50,nintmax=10,nbinmax=500,nitmax=10)
* sa1end
      DIMENSION test(ndismax),test_mix(ndismax),devbin(ndismax,nbinmax)
      DIMENSION devbin_mix(ndismax,nbinmax),resl_nc(nitmax)
      DIMENSION resl_cc(nitmax),rnorm(ndismax,nbinmax,nitmax)
      DIMENSION rnorm_mix(ndismax,nbinmax,nitmax),nbintot(ndismax),
     &  nbintest(ndismax)
      COMMON/abdist/distr_estrinf(ndismax,nintmax+1),bin_width(ndismax,
     &  nintmax),distr_local(ndismax,nbinmax,nitmax),distr_loc_mix
     &  (ndismax,nbinmax,nitmax),dev_local(ndismax,nbinmax,nitmax),
     &  dev_loc_mix(ndismax,nbinmax,nitmax),tail_local(ndismax,nitmax),
     &  tail_loc_mix(ndismax,nitmax)
      COMMON/abidis/ncallbin(ndismax,nbinmax,nitmax),
     &  ncallbin_mix(ndismax,nbinmax,nitmax),nbin_number(ndismax,
     &  nintmax),nbin_sum(ndismax,nintmax),nsubint(ndismax),ndistr,
     &  idistr,it1,it2,init
      COMMON/abcdis/string(ndismax)
      COMMON/abresl/resl(10),standdevl(10)
*agg
      COMMON/abflat/rmaxfxn,rmaxfxn_1it,rmaxfxn_2it,rmaxfxn_cc_1it,
     &  rmaxfxn_cc_2it,rmaxfxn_nc_1it,rmaxfxn_nc_2it,scalemax
      COMMON/abifla/itmx,novermax,iflat,iseed,istorvegas,istormom,iterm,
     &  ijetset,interf
      COMMON/abfla2/irepeat,nevent,nflevts
*aggend
C beam
      COMMON/abibea/ibeam
      COMMON/abxbea/x1beam,x2beam
C beam
* em:esponenti
* em2
      COMMON/rexp/rexp5,rexp6,rexpsm
* endem2
* em:end


* sa1
C for new reading system
      CHARACTER*80 pluto
      CHARACTER*1 charint1
      CHARACTER*2 charint2
      CHARACTER*3 charint3
      COMMON/abread/pluto

      DIMENSION auread(nintmax+1)
      DIMENSION iauread(nintmax)

Cqedps
      double precision ainmas
*     Mass of the electron, used by QEDPS
      PARAMETER (ainmas=0.51099907d-3)
      double precision plptn
      INTEGER nlptn,nptcl
* QEDPS Common Block
      COMMON/qplist/plptn(10,1000),nlptn(10,1000),nptcl

* sa1end


      EXTERNAL fxn,gammln

      DATA rmw/80.350d0/, rmz/91.1867d0/, rmt/175.d0/, rmc/1.35d0/,
     &     rmtau/1.78d0/, rmb_run/2.7d0/, rmu/0.005d0/, rmd/0.01d0/,
     &     rme/0.51099906d-3/, rms/0.15d0/, rmmu/0.105d0/, rmb/4.7d0/,
     &     gamw/0.2042774d+01/, gamz/0.2440851d+01/, gamh/1.278d-03/,
     &     gf/1.16637d-05/, alfainv/128.07d0/,
     &     alfas_cc/0.1255d0/, alfas_nc/0.1230d0/,
     &     s2w/0.231030912451068d0/,rmsus/1000.d0/
C beam
     &     ,x1beam/1.d0/,x2beam/1.d0/
C beam
* em2
      DATA rexp5/0.7d0/,rexp6/0.7d0/,rexpsm/1.3d0/
     &     ,sepmass1low/20.d0/,sepmass1high/60.d0/
     &     ,sepmass2low/20.d0/,sepmass2high/60.d0/
* endem2

      DATA i3e/0/, i4e/0/, i3q/0/, i5q/0/, iqu/0/, i34e/0/, i56ve/0/,
     &     ibbveve/0/, iid/0/, pi/3.141592653589793238462643d0/,
     &     r/0.2d0/, irepeat/0/, interf/0/, isusy/0/, ichcj/0/, iha/0/
*elalfa
Calfa
     &     ,ialftsw/0/, ialfar/0/
Calfaend
*elalfaend
* sa2
     &     , ireadinput/0/

C  ireadinput  1= read normali, 0 file di input

     &     , iqu/0/
C       iqu     ! yes/no QCD diagrams for 4-quarks NC
     &    , ifloop/0/

* sa2end


*begin{EZ}
      COMMON/abfloop/ifloop
*elalfa
Calfa
     &      ,ialftsw,ialfar
Calfaend
*elalfaend
*end{EZ}


****
* imix=2 CC+interf.NC (only if interf.=1), =1 CC (or Higgs),
*       =0 interf., =-1 NC, =-2 NC + interf. CC(or Higgs)
* iccnc  1=CC only, 2=NC only, 3= CC+NC +interf (not only for mixed
*  processes where there is a read, but also defined in initialize
*  for all proc, and =4 for b proc.
* icc  =1 calcola solo CC, =0 solo NC, =-1 CC+NC (in ME calcul.)
****




* sa1

      IF (ireadinput.EQ.0) THEN
        PRINT*, 'name of the input file ?'
        READ (*,'(a)') pluto       ! name of input file
        PRINT*, '                          INPUT FROM FILE '//pluto


        CALL rread('e_cm',e_cm,1)      ! centre of mass energy (GeV)
        CALL iread('iproc',iproc,1)
                   !selects the kind of process (see subr. INITIALIZE)
        CALL iread('ich',ich,1)
                    ! selects the channel (see subroutine INITIALIZE)
        CALL iread('imass',imass,1)
                                  !imass=0 massless, imass=1 massive
*el8
        CALL iread('icc3',icc3,1)
                   !selects: 1=cc3, 0=all, -1=all-cc3,
                   !         2=(all-cc3)+Int[all-cc3,cc3]
*endel8
        IF (iproc.GE.6.AND.iproc.LE.8) THEN    !in mixed processes:
          CALL iread('iccnc',iccnc,1)
                              ! 1=CC only, 2=NC only, 3= CC+NC +interf
        ENDIF
*el8
        CALL iread('izz',izz,1)
                   !selects: 1=NC02, 0=all, -1=all-NC02,
                   !         2=(all-NC02)+Int[all-NC02,NC02]
        CALL iread('izg',izg,1)
                   !selects: 1=Z-gamma, 0=all, -1=all-(Z-gamma),
                   !         2=all-(Z-gamma)+Int[all-(Z-gamma),(Z-gamma)]
        CALL iread('izg34',izg34,1)
                   !selects: 1=Z56-gamma34, 0=all,-1=all-(Z56-gam34),
                   !         2=all-(Z56-gam34)+Int[all-(Z56-gam34),(Z56-gam34)]
        CALL iread('izg56',izg56,1)
                   !selects: 1=Z34-gamma56, 0=all,-1=all-(Z34-gam56),
                   !         2=all-(Z34-gam56)+Int[all-(Z34-gam56),(Z34-gam56)]
        CALL iread('inc08',inc08,1)
                   !selects: 1=NC08, 0=all, -1=all-NC08,
                   !         2=all-NC08+Int[all-NC08,NC08]
        CALL iread('igamgam',igamgam,1)
                   !selects: 1=gamma-gamma, 0=all, -1=all-(gamma-gamma),
                   !         2=all-(gamma-gamma)+Int[all-(gam-gam),(gam-gam)]
        CALL iread('itch',itch,1)
                   !selects: 1=t-channel, 0=all, -1=s-channel,
                   !         2=s-channel+Int[s-channel,t-channel]
*endel8

        CALL iread('ips_cc',ips_cc,1)
                   ! CC (or Higgs signal) phase space :
                   ! 1=double resonant, 2=single resonant (34),
                   ! 3=single resonant (56), 4=non resonant
        CALL iread('ips_nc',ips_nc,1)
                   ! NC phase space:
                   ! 1=double resonant, 2=single resonant (34),
                   ! 3=single resonant (56), 4=non resonant

        CALL iread('ismallangle',ismallangle,1)
                   ! 0=no small angle,  1=e- near the beam (180 degrees)
                   ! 2= e- near 180 degrees,  e+ near 0 degrees
Calfa
        CALL iread('ialftsw',ialftsw,1)
                   !yes/no  alfa running for t channel
Calfaend
*elalfa
        CALL iread('ialfar',ialfar,1)
                   !yes/no  alfa running for all gamma couplings
*elalfaend
        CALL iread('isr',isr,1)         ! yes/no ISR
        CALL iread('ibeam',ibeam,1)       !yes/no beamstrahlung
        CALL iread('icoul',icoul,1)       ! yes/no Coulomb corrections
        CALL iread('istrcor',istrcor,1)
                     ! yes/no 'naive' QCD corrections (also for widths)
        CALL iread('ianc',ianc,1)        ! yes/no anomalous couplings
        IF(ianc.EQ.1)THEN
          CALL rread('delz',delz,1)
          CALL rread('xf',xf,1)
          CALL rread('xz',xz,1)
          CALL rread('yf',yf,1)
          CALL rread('yz',yz,1)
          CALL rread('zz',zz,1)       !anomalous couplings parameters
        ENDIF
* sa2
        IF (iproc.EQ.2.OR.iproc.EQ.4.OR.
     &         (iproc.EQ.7.AND.iccnc.NE.2)) THEN
*sa2end
          CALL iread('ifloop',ifloop,1)
* sa2
        ENDIF
*sa2end
        CALL iread('ipr',ipr,1)
                    ! widths: 0/1 Z,W fixed/running 2 Z fi, W run
        CALL iread('iswgcomp',iswgcomp,1)
                      ! yes/no s2w and g computed (0= DATA value)
        CALL iread('igwcomp',igwcomp,1)
        CALL iread('igzcomp',igzcomp,1)
        CALL iread('ighcomp',ighcomp,1)
                        ! yes/no W,Z,H width computed (0= DATA val.)

        CALL iread('icut',icut,1)        ! yes/no cuts
        IF(icut.EQ.1)THEN
          CALL rread('e_min',e_min,4)     ! 4 energy lower cuts (GeV)
          CALL rread('e_max',e_max,4)     ! 4 energy upper cuts (GeV)
          CALL rread('rm_min',rm_min,6)
                                 ! 6 invariant mass lower limits (GeV)
                                 ! (34, 35, 36, 45, 46, 56)
          CALL rread('rm_max',rm_max,6)
                                 ! 6 invariant mass upper limits (GeV)
          CALL rread('pt_min',pt_min,4)
                                ! 4 transverse momenta lower cuts (GeV)
          CALL rread('pt_max',pt_max,4)
                                ! 4 transverse momenta upper cuts (GeV)
          CALL iread('icos',icos,1) ! angular cuts in deg (0) or cos (1)
*el6
          CALL iread('iext',iext,4) ! 0/1=internal/external  for 4
                                    ! particle beam angular region
*endel6
          CALL rread('thbeam_min',thbeam_min,4)
                         ! 4 particle-beam angle lower (in degrees) cuts
          CALL rread('thbeam_max',thbeam_max,4)
                         ! 4 particle-beam angle upper (in degrees) cuts
          CALL rread('thsep_min',thsep_min,6)
                        ! 6 particl-particl angle lower(in degrees) cuts
          CALL rread('thsep_max',thsep_max,6)
                        ! 6 particl-particl angle upper(in degrees) cuts
        ENDIF
        CALL iread('idistr',idistr,1)      ! yes/no distributions
        IF(idistr.EQ.1)THEN
          CALL iread('ndistr',ndistr,1)    ! number of distributions
          IF(ndistr.GT.ndismax)THEN
            PRINT*,'            '
            PRINT*,'           ************* ERROR: **************'
            PRINT*,'Number of distributions cannot exceed'
            PRINT*,'NDISMAX given as parameter'
            PRINT*,'            '
            STOP
          ENDIF
          DO i=1,ndistr
            CALL convert(i,charint1,charint2,charint3)
            IF (i.LE.9) THEN
              CALL iread('nsubint('//charint1//')',nsubint(i),1)
            ELSEIF (i.LE.99) THEN
              CALL iread('nsubint('//charint2//')',nsubint(i),1)
            ELSEIF (i.LE.999) THEN
              CALL iread('nsubint('//charint3//')',nsubint(i),1)
            ENDIF
                                     !number of sub-intervals with
                                     !         different binning
            IF(nsubint(i).GT.nintmax)THEN
              PRINT*,'            '
              PRINT*,'           ************* ERROR: **************'
              PRINT*,'Number of sub-intervals cannot exceed'
              PRINT*,'NINTMAX given as parameter'
              PRINT*,'            '
              STOP
            ENDIF
            IF (i.LE.9) THEN
              CALL rread('distr_estrinf('//charint1//')',
     &                    auread,nsubint(i)+1)
            ELSEIF (i.LE.99) THEN
              CALL rread('distr_estrinf('//charint2//')',
     &                    auread,nsubint(i)+1)
            ELSEIF (i.LE.999) THEN
              CALL rread('distr_estrinf('//charint3//')',
     &                    auread,nsubint(i)+1)
            ENDIF
            DO j=1,nsubint(i)+1
              distr_estrinf(i,j)=auread(j)
            ENDDO
                     !lower limits of each
                     !subint (which coincide with the upper limit of the
                     ! previous one) + upper limit of the last subint
            IF (i.LE.9) THEN
              CALL iread('nbin_number('//charint1//')',
     &                    iauread,nsubint(i))
            ELSEIF (i.LE.99) THEN
              CALL iread('nbin_number('//charint2//')',
     &                    iauread,nsubint(i))
            ELSEIF (i.LE.999) THEN
              CALL iread('nbin_number('//charint3//')',
     &                    iauread,nsubint(i))
            ENDIF
            DO j=1,nsubint(i)
              nbin_number(i,j)=iauread(j)
            ENDDO
                                       !number of bins in each subint
            DO j=1,nsubint(i)
              nbintest(i)=nbintest(i)+nbin_number(i,j)
            ENDDO !j
            IF(nbintest(i).GT.nbinmax)THEN
              PRINT*,'            '
              PRINT*,'           ************* ERROR: **************'
              PRINT*,'total number of bins cannot exceed'
              PRINT*,'NBINMAX given as parameter'
              PRINT*,'            '
              STOP
            ENDIF
          ENDDO !i
        ENDIF
        CALL iread('iflat',iflat,1)    ! yes/no flat event generation
        IF(iflat.EQ.1)THEN
          CALL rread('scalemax',scalemax,1)
                                 ! scale factor for the maximum
          CALL iread('istorvegas',istorvegas,1)
                                 ! yes/no VEGAS data stored
          CALL iread('irepeat',irepeat,1)
                         ! 0=normal;1=repeat 2nd only;2=rep.
                         !                     with nflevts fixed
                         ! 1=repeat only second iteration using vegas
                         !         data stored in previous run,
                         ! 2=repeat, generating an exact number nflevts
                         !     of flatevents using vegas data stored
                         !         in previous run
          IF(irepeat.EQ.2)THEN
            CALL iread('nflevts',nflevts,1)
                               ! number of events to be generated as
                               !               described above
          END IF
          CALL iread('istormom',istormom,1)
                  ! yes/no momenta of flat events written in .dat files
          CALL iread('ijetset',ijetset,1)    ! yes/no call to Jetset
          IF(iproc.GE.6.AND.iproc.LE.8)THEN
            CALL iread('interf',interf,1)
                                ! 0= mix. interference added to NC;
                                !    1=  added to CC
          ENDIF
        ENDIF
* em2
*        call rread('rexp5',rexp5,1)
*        call rread('rexp6',rexp6,1)
*        PRINT *,'rexp5=',rexp5,'  rexp6=',rexp6
*        call rread('ism34',ism34,1)
*        call rread('ism56',ism56,1)
*        PRINT *,'ism34 =',ism34,' ism56 =',ism56
* endem2
        CALL rread('acc',acc,1)         ! integration accuracy
        CALL iread('iterm',iterm,1)       ! yes/no thermalization
        CALL iread('ncall_term',ncall_term,1)
                                ! thermalization calls per iteration
        CALL iread('itmx_term',itmx_term,1)  ! thermalization iterations
        CALL iread('ncall',ncall,1)    ! integration calls per iteration
        CALL iread('itmx',itmx,1)
                  ! integration iterations (2 for flat event generation)

        IF(iproc.GE.33)THEN
          CALL rread('rmb',rmb,1)           ! quark b mass (GeV)
* el1
          CALL rread('rmc',rmc,1)           ! quark c mass (GeV)
          CALL rread('rmtau',rmtau,1)         ! tau     mass (GeV)
*
          CALL rread('rmb_run',rmb_run,1)   ! quark b mass running(GeV)
          CALL rread('rmc_run',rmc_run,1)   ! quark c mass running(GeV)
* endel1
          CALL rread('gamh',gamh,1)          ! higgs width
          CALL rread('alfas_h',alfas_h,1)       ! alfas at Higgs mass
          CALL iread('icch',icch,1)
                   ! 1=Higgs signal, 2=Background, 3=Higgs+Backg+interf
          IF(icch.NE.2)THEN
            CALL iread('isusy',isusy,1)
            IF(isusy.EQ.0)THEN
* el1
              IF((iproc.EQ.39.OR.iproc.EQ.43.OR.iproc.EQ.51).AND.
     &          icch.EQ.1)THEN
* endel1
                CALL iread('iha',iha,1)    ! 1=all, 2=hZ
              ENDIF
              CALL rread('rmh',rmh,1)       ! Higgs mass (GeV)
            ELSE
* el1
              IF((iproc.EQ.39.OR.iproc.EQ.43.OR.iproc.EQ.51).AND.
     &          icch.EQ.1)THEN
* endel1
                CALL iread('iha',iha,1)
                        ! 1=all, 2=h, 3=A, 4=h-stralhung, 5=A-strahlung
              ENDIF
              CALL iread('irmhcomp',irmhcomp,1)  ! yes/no rmh computed
              IF(irmhcomp.EQ.1)THEN
                CALL rread('rma',rma,1)       ! Higgs 'A' mass (GeV)
                CALL rread('tgb',tgb,1)       ! tan(beta)
                CALL iread('iloop',iloop,1)
                               ! 1=1-loop, 2=Carena et al. corrections
                IF(iloop.EQ.2)THEN
                  CALL iread('imixing',imixing,1)
                               !1=no, 2=maximal, 3=typical, 4=other
                  IF(imixing.EQ.4)THEN
                    CALL rread('At',at,1)
                    CALL rread('Ab',ab,1)
* el1
                    CALL rread('rmyou',rmyou,1)
* endel1
                  ENDIF
                ENDIF
              ELSE
                CALL rread('rmh',rmh,1)
                CALL rread('rma',rma,1)
                CALL rread('tgb',tgb,1)
              ENDIF
            ENDIF
          ENDIF
        ENDIF


      ELSE

* sa1end

        READ*,e_cm      ! centre of mass energy (GeV)
        READ*,iproc    !selects the kind of process (see subr. INITIALIZE)
        READ*,ich       ! selects the channel (see subroutine INITIALIZE)
        READ*,imass     !imass=0 massless, imass=1 massive
*el8
        READ*,icc3     !1=cc3, 0=all, -1=all-cc3, 2=all-cc3+Int[all-cc3,cc3]
*endel8
        IF (iproc.GE.6.AND.iproc.LE.8) THEN    !in mixed processes:
          READ*,iccnc     ! 1=CC only, 2=NC only, 3= CC+NC +interf
        ENDIF
*el8
        READ*,izz      !1=ZZ, 0=all, -1=all-ZZ, 2=all-ZZ+Int[all-ZZ,ZZ]
        READ*,izg      !1=ZG, 0=all, -1=all-ZG, 2=all-ZG+Int[all-ZG,ZG]
        READ*,izg34    !1=Z56-G34, 0=all, -1=all-(Z56-G34),
                       !2=all-(Z56-G34)+Int[all-(Z56-G34),(Z56-G34)]
        READ*,izg56    !1=Z34-G56, 0=all, -1=all-(Z34-G56),
                       !2=all-(Z34-G56)+Int[all-(Z34-G56),(Z34-G56)]
        READ*,inc08    !1=NC08, 0=all,-1=all-NC08, 2=all-NC08+Int[all-NC08,NC08]
        READ*,igamgam  !1=gam-gam, 0=all,-1=all-GG, 2=all-GG+Int[all-GG,GG]
        READ*,itch     !1=t-channel, 0=all,-1=all-(t-channel),
                       !2=all-(t-channel)+Int[all-(t-channel),t-channel]
*endel8
Ctest

        READ*,ips_cc      ! CC (or Higgs signal) phase space :
                          ! 1=double resonant, 2=single resonant (34),
                          ! 3=single resonant (56), 4=non resonant
        READ*,ips_nc      ! NC phase space:
                          ! 1=double resonant, 2=single resonant (34),
                          ! 3=single resonant (56), 4=non resonant

Ctestend

        READ*,ismallangle ! 0=no small angle,
                          ! 1=e- near the beam (180 degrees)
                          ! 2= e- near 180 degrees,  e+ near 0 degrees
Calfa
        READ*,ialftsw  !yes/no  alfa running for t channel
Calfaend
*elalfa
        READ*,ialfar  !yes/no  alfa running for all gamma couplings
*elalfaend
        READ*,isr         ! yes/no ISR
        READ*,ibeam       !yes/no beamstrahlung
        READ*,icoul       ! yes/no Coulomb corrections
        READ*,istrcor  ! yes/no 'naive' QCD corrections (also for widths)
        READ*,ianc        ! yes/no anomalous couplings
        IF(ianc.EQ.1)THEN
          READ*, delz,xf,xz,yf,yz,zz !anomalous couplings parameters
        ENDIF
* sa2
        IF (iproc.EQ.2.OR.iproc.EQ.4.OR.
     &         (iproc.EQ.7.AND.iccnc.NE.2)) THEN
*sa2end

          READ*,ifloop
* sa2
        ENDIF
*sa2end
        READ*,ipr         ! widths: 0/1 Z,W fixed/running 2 Z fi, W run
        READ*,iswgcomp    ! yes/no s2w and g computed (0= DATA value)
        READ*,igwcomp,igzcomp,ighcomp
                        ! yes/no W,Z,H width computed (0= DATA val.)
        READ*,icut        ! yes/no cuts
        IF(icut.EQ.1)THEN
          READ*,e_min     ! 4 energy lower cuts (GeV)
          READ*,e_max     ! 4 energy upper cuts (GeV)
          READ*,rm_min    ! 6 invariant mass lower limits (GeV)
                          ! (34, 35, 36, 45, 46, 56)
          READ*,rm_max    ! 6 invariant mass upper limits (GeV)
          READ*,pt_min    ! 4 transverse momenta lower cuts (GeV)
          READ*,pt_max    ! 4 transverse momenta upper cuts (GeV)
          READ*,icos      ! angular cuts in deg (0) or cos (1)
*el5
          READ*,iext      ! 0/1=internal/external for 4 particle beam
                          !  angular region
*endel5
          READ*,thbeam_min! 4 particle-beam angle lower (in degrees) cuts
          READ*,thbeam_max! 4 particle-beam angle upper (in degrees) cuts
          READ*,thsep_min ! 6 particl-particl angle lower(in degrees) cuts
          READ*,thsep_max ! 6 particl-particl angle upper(in degrees) cuts
        ENDIF
        READ*,idistr      ! yes/no distributions
        IF(idistr.EQ.1)THEN
          READ*,ndistr    ! number of distributions
          IF(ndistr.GT.ndismax)THEN
            PRINT*,'            '
            PRINT*,'           ************* ERROR: **************'
            PRINT*,'Number of distributions cannot exceed'
            PRINT*,'NDISMAX given as parameter'
            PRINT*,'            '
            STOP
          ENDIF
          DO i=1,ndistr
            READ*,nsubint(i) !number of sub-intervals with
                             !                        different binning
            IF(nsubint(i).GT.nintmax)THEN
              PRINT*,'            '
              PRINT*,'           ************* ERROR: **************'
              PRINT*,'Number of sub-intervals cannot exceed'
              PRINT*,'NINTMAX given as parameter'
              PRINT*,'            '
              STOP
            ENDIF

            READ*,(distr_estrinf(i,j),j=1,nsubint(i)+1) !lower limits of each
                                                        ! subint (which coincide with the upper limit of the
                                                        ! previous one) + upper limit of the last subint
            READ*,(nbin_number(i,j),j=1,nsubint(i)) !number of bins in each
*                                                  subint
            DO j=1,nsubint(i)
              nbintest(i)=nbintest(i)+nbin_number(i,j)
            ENDDO !j
            IF(nbintest(i).GT.nbinmax)THEN
              PRINT*,'            '
              PRINT*,'           ************* ERROR: **************'
              PRINT*,'total number of bins cannot exceed'
              PRINT*,'NBINMAX given as parameter'
              PRINT*,'            '
              STOP
            ENDIF
          ENDDO !i
        ENDIF
        READ*,iflat        ! yes/no flat event generation
        IF(iflat.EQ.1)THEN
          READ*,scalemax   ! scale factor for the maximum
          READ*,istorvegas ! yes/no VEGAS data stored
          READ*,irepeat    ! 0=normal;1=repeat 2nd only;2=rep.
                           !                     with nflevts fixed
                           ! 1=repeat only second iteration using vegas
                           !         data stored in previous run,
                           ! 2=repeat, generating an exact number nflevts
                           !     of flatevents using vegas data stored
                           !         in previous run
          IF(irepeat.EQ.2)THEN
            READ*, nflevts ! number of events to be generated as
                           !               described above
          END IF
          READ*,istormom   ! yes/no momenta of flat events written in
                           !                          .dat files
          READ*,ijetset    ! yes/no call to Jetset
          IF(iproc.GE.6.AND.iproc.LE.8)THEN
            READ*,interf   ! 0= mix. interference added to NC;
                           !                       1=  added to CC
          ENDIF
        ENDIF

        READ*,acc         ! integration accuracy
        READ*,iterm       ! yes/no thermalization
        READ*,ncall_term  ! thermalization calls per iteration
        READ*,itmx_term   ! thermalization iterations
        READ*,ncall       ! integration calls per iteration
        READ*,itmx  ! integration iterations (2 for flat event generation)

        IF(iproc.GE.33)THEN
          READ*,rmb           ! quark b mass (GeV)
* el1
          READ*,rmc           ! quark c mass (GeV)
          READ*,rmtau         ! tau     mass (GeV)
*
          READ*,rmb_run       ! quark b mass running(GeV)
          READ*,rmc_run       ! quark c mass running(GeV)
* endel1
          READ*,gamh          ! higgs width
          READ*,alfas_h       ! alfas at Higgs mass
          READ*,icch  ! 1=Higgs signal, 2=Background, 3=Higgs+Backg+interf
          IF(icch.NE.2)THEN
            READ*,isusy
            IF(isusy.EQ.0)THEN
* el1
              IF((iproc.EQ.39.OR.iproc.EQ.43.OR.iproc.EQ.51).AND.
     &          icch.EQ.1)THEN
* endel1
                READ*,iha    ! 1=all, 2=hZ
              ENDIF
              READ*,rmh       ! Higgs mass (GeV)
            ELSE
* el1
              IF((iproc.EQ.39.OR.iproc.EQ.43.OR.iproc.EQ.51).AND.
     &          icch.EQ.1)THEN
* endel1
                READ*,iha  ! 1=all, 2=h, 3=A, 4=h-stralhung, 5=A-strahlung
              ENDIF
              READ*,irmhcomp  ! yes/no rmh computed
              IF(irmhcomp.EQ.1)THEN
                READ*,rma       ! Higgs 'A' mass (GeV)
                READ*,tgb       ! tan(beta)
                READ*,iloop     ! 1=1-loop, 2=Carena et al. corrections
                IF(iloop.EQ.2)THEN
                  READ*,imixing    !1=no, 2=maximal, 3=typical, 4=other
                  IF(imixing.EQ.4)THEN
                    READ*,at
                    READ*,ab
* el1
                    READ*,rmyou
* endel1
                  ENDIF
                ENDIF
              ELSE
                READ*,rmh
                READ*,rma
                READ*,tgb
              ENDIF
            ENDIF
          ENDIF
        ENDIF


        WRITE (*,*) 'ifloop = ',ifloop
* sa1
C fine dei read normali
      ENDIF
* sa1end


** input check
* sa2
      IF(itmx_term.GT.nitmax)THEN
        PRINT*,'            '
        PRINT*,'           ************* ERROR: **************'
        PRINT*,'Number of thermalization iterations cannot exceed'
        PRINT*,'NITMAX given as parameter'
        PRINT*,'            '
        STOP
      ENDIF
* sa2end

      IF(itmx.GT.nitmax)THEN
        PRINT*,'            '
        PRINT*,'           ************* ERROR: **************'
        PRINT*,'Number of integration iterations cannot exceed'
        PRINT*,'NITMAX given as parameter'
        PRINT*,'            '
        STOP
      ENDIF
* el1
      IF(e_cm.NE.360.d0.AND.e_cm.NE.500.d0.AND.e_cm.NE.800.d0.
     &   and.ibeam.EQ.1)THEN
        PRINT*,'The beamstrahlung via CIRCE is available only'
        PRINT*,'for e_cm=(360, 500, 800) GeV'
        STOP
      ENDIF
* endel1
* em2
      IF((ismallangle.GT.0).AND.(iproc.NE.2).AND.(iproc.NE.4)
     &    .AND.(iproc.NE.7).AND.(iproc.NE.21).AND.(iproc.NE.26)
     &    .AND.(iproc.NE.28).AND.(iproc.NE.30))THEN
        PRINT*,'            '
        PRINT*,'           ************* ERROR: **************'
        PRINT*,'Small angle phase space requires an electron'
        PRINT*,'or positron in the final state.'
        PRINT*,'iproc=',iproc,' ismallangle=', ismallangle
        PRINT*,'            '
        STOP
      ENDIF
      IF((ismallangle.GT.0).AND.(imass.EQ.0))THEN
        PRINT*,'            '
        PRINT*,'           ************* ERROR: **************'
        PRINT*,'Small angle phase space requires fully'
        PRINT*,'massive matrix elements (imass=1).'
        PRINT*,'            '
        STOP
      ENDIF
Calfa
      IF((ialftsw.NE.0).AND.(ialftsw.NE.1)) THEN
        PRINT*,'            '
        PRINT*,'           ************* ERROR: **************'
        PRINT*,'              ialftsw has to be 0 or 1'
        PRINT*,'            '
        STOP
      ENDIF
      IF((ialftsw.NE.0).AND.(iproc.NE.2.AND.iproc.NE.4.AND.iproc.NE.7))
     &  THEN
        PRINT*,'            '
        PRINT*,'           ************ WARNING: *************'
        PRINT*,'       ialftsw can be different from zero only for  '
        PRINT*,'        CC or MIX processes with a final e- or e+  '
        PRINT*,'                 ialftsw set to 0 '
        ialftsw=0
      ENDIF
Calfaend
*elalfa
      IF((ialfar.NE.0).AND.(ialfar.NE.1)) THEN
        PRINT*,'            '
        PRINT*,'           ************* ERROR: **************'
        PRINT*,'              ialfar has to be 0 or 1'
        PRINT*,'            '
        STOP
      ENDIF
      IF((ialfar.EQ.1).AND.(ialftsw.EQ.1)) THEN
        PRINT*,'            '
        PRINT*,'           ************* ERROR: **************'
        PRINT*,'              if ialftsw=1 then ialfar has to be 0'
        PRINT*,'            '
        STOP
      ENDIF
*elalfaend
* endem2

      IF((imass.EQ.1).AND.(iproc.GT.32))THEN
        PRINT*,'            '
        PRINT*,'           ************* ERROR: **************'
        PRINT*,'The matrix element for the selected process is not'
        PRINT*,' fully massive: only b quarks, taus and virtual tops'
        PRINT*,' masses  are included.'
        PRINT*,' If fully massive computations are required, '
        PRINT*,' processes 1 to 32 have to be selected'
        PRINT*,' Otherwise imass must be set to 0'
        PRINT*,'            '
        STOP
      ENDIF

      IF((izz.EQ.1.OR.izg.EQ.1.OR.izg34.EQ.1.OR.izg56.EQ.1.OR.
     &   inc08.EQ.1.OR.igamgam.EQ.1).AND.(iproc.LE.5.OR.imass.EQ.0))THEN
        PRINT*,'            '
        PRINT*,'           ************* ERROR: **************'
        PRINT*,'The options izz,izg,izg34,izg56,inc08,igamgam are'
        PRINT*,'available only for Mixed and NC massive processes'
        PRINT*,'            '
        STOP
      ENDIF

* sa2
      IF((izz.EQ.1.OR.izg.EQ.1.OR.izg34.EQ.1.OR.izg56.EQ.1.OR.
     &   inc08.EQ.1.OR.igamgam.EQ.1).AND.(iproc.GT.5.AND.iproc.LT.9).
     &   and.iccnc.NE.2)THEN
        PRINT*,'            '
        PRINT*,'           ************* WARNING: **************'
        PRINT*,'The options izz,izg,izg34,izg56,inc08,igamgam require'
        PRINT*,' the evaluation of NC diagrams only.'
        PRINT*,' iccnc =2 is therefore required for Mixed processes'
        PRINT*,'             '
        PRINT*,' iccnc changed to 2 '
        iccnc=2
        PRINT*,'            '
      ENDIF

      IF(iflat.EQ.1.AND.itmx.NE.2)THEN
        PRINT*,'            '
        PRINT*,'           ************* ERROR: **************'
        PRINT*,'Flat events generation needs ITMX =2'
        PRINT*,'            '
        STOP
      ENDIF
      IF (ifloop.EQ.1.AND.ipr.NE.2) THEN
        PRINT*,'            '
        PRINT*,'           ************* WARNING: **************'
        PRINT*,'  ifloop=1 requires ipr=2.  ipr changed to 2'
        ipr=2
        PRINT*,'            '
      ENDIF
* endsa2

** input check end

C beam
* el1
      IF(ibeam.EQ.1)THEN
        IF(e_cm.EQ.360.d0)THEN
          CALL circes (0.d0,0.d0,360.d0,2,1,1996 09 02,0)
        ELSE IF(e_cm.EQ.500.d0)THEN
          CALL circes (0.d0,0.d0,500.d0,2,1,1996 09 02,0)
        ELSE IF(e_cm.EQ.800.d0)THEN
          CALL circes (0.d0,0.d0,800.d0,2,1,1996 09 02,0)
        ENDIF
      ENDIF
* endel1
C beam

* sa1

Cqedps
      IF ( isr .EQ.2) THEN
* Intialize QEDPS
        q2max=e_cm**2
        CALL qpinit(q2max,ainmas,iranseed)
      ENDIF
Cqedpsend
* sa1end



      s_col=e_cm**2
* el1
*      rmc_run=rmc
* endel1
      rmw2=rmw**2
      rmz2=rmz**2
      rmt2=rmt**2
      rmb2=rmb**2
      rmh2=rmh**2
      IF(iswgcomp.EQ.1)THEN
        s2w=1.d0-rmw2/rmz2
        g2=4.d0*sqrt(2.d0)*gf*rmw2
        alfainv=4.d0*pi/(s2w*g2)
      ENDIF
      sw=sqrt(s2w)
      rc2w=1.d0-s2w
      rcw=sqrt(rc2w)
      rcotw=rcw/sw
      IF(isusy.EQ.1)THEN
        rma2=rma**2
        rmsus2=rmsus**2
        bet=atand(tgb)
        rcb=cosd(bet)
        sb=sind(bet)
        s2b=sind(2.d0*bet)
        rc2b=cosd(2.d0*bet)
        tg2b=tand(2.d0*bet)
        IF(irmhcomp.EQ.1)THEN
          alfa_t=alfas_nc/(1+23.d0*alfas_nc*log(rmt2/rmz2)/(12.d0*pi))
          rmt_run=rmt/(1.d0+4.d0*alfa_t/(3.d0*pi))
          IF(iloop.EQ.1)THEN
            epsi=3.d0*rmt_run**4*log(1.d0+rmsus2/rmt2)/
     &           (2.d0*pi*s2w*rmw2*sb**2*alfainv)
            rmh2=0.5d0*(rmz2+rma2+epsi-sqrt((rmz2+rma2+epsi)**2-4.d0*
     &           (rma2*(rmz2*rc2b**2+epsi*sb**2)+epsi*rmz2*rcb**2)))
          ELSE IF(iloop.EQ.2)THEN
* Neutral CP-even Higgs corrections (Carena et al.):
* Tutte le variabile sono alla scala Mt
            IF(imixing.EQ.1)THEN
              at=0.d0
              ab=0.d0
* el1
              rmyou=0.d0
* endel1
            ELSE IF(imixing.EQ.2)THEN
              at=sqrt(6.d0)*rmsus
              ab=at
* el1
              rmyou=0.d0
* endel1
            ELSE IF(imixing.EQ.3)THEN
              at=rmsus
              ab=at
* el1
              rmyou=-rmsus
* endel1
            ENDIF
            fourpi=4.d0*pi
            fourpi2=fourpi**2
* el1
            rmyouq=rmyou**2
* endel1
            rmtau2=rmtau**2
            rmt2_run=rmt_run**2
            rmb2_run=rmb_run**2
            sb2=sb**2
            rcb2=rcb**2
            g1=5.d0*fourpi/(3.d0*(1.d0-s2w)*alfainv)
            g1=g1/(1.d0+53.d0*g1*log(rmz/rmt)/(120.d0*pi**2))
            g2=fourpi/(s2w*alfainv)
            g2=g2/(1.d0-11.d0*g2*log(rmz/rmt)/(24.d0*pi**2))
            g3=fourpi*alfa_t
            v2=4.d0*rmw2/g2
            htq=2.d0*rmt2_run/(v2*sb2)
            hbq=2.d0*rmb2_run/(v2*rcb2)
            htauq=2.d0*rmtau2/(v2*rcb2)
            t=2.d0*log(rmsus/rmt)
            xt=2.d0*at**2*(1.d0-at**2/(12.d0*rmsus2))/rmsus2
            xb=2.d0*ab**2*(1.d0-ab**2/(12.d0*rmsus2))/rmsus2
* el1
            atb=(-6.d0*rmyouq/rmsus2-(rmyouq-ab*at)**2/(rmsus2**2)+
     &          3.d0*(at+ab)**2/rmsus2)/6.d0
* endel1
            IF(rma.GT.rmt)THEN
              tgb=tgb*(1.d0+3.d0*(htq-hbq)*log(rma/rmt)/fourpi2)
              sb=sin(atan(tgb))
              sb2=sb**2
              rcb=cos(atan(tgb))
              rcb2=rcb**2
              s2b=sin(2.d0*atan(tgb))
              rc2b=cos(2.d0*atan(tgb))
              bet=atand(tgb)
              tg2b=tand(2.d0*bet)
              htq=2.d0*rmt2_run/(v2*sb2)
              hbq=2.d0*rmb2_run/(v2*rcb2)
              htauq=2.d0*rmtau2/(v2*rcb2)
            ENDIF
* el1: rmu -> rmyou
            rl1=0.25d0*(g1+g2)*(1.d0-6.d0*hbq*t/fourpi2)+
     &        6.d0*hbq**2*(t+xb*0.5d0+(1.5d0*hbq+0.5d0*htq-8.d0*g3)*
     &        (xb*t+t**2)/fourpi2)/fourpi2-3.d0*htq**2*rmyouq**2
     &        *(1.d0+(9.d0*htq-5.d0*hbq-16.d0*g3)*t/fourpi2)/
     &        (96.d0*pi**2*rmsus2**2)
            rl2=0.25d0*(g1+g2)*(1.d0-6.d0*htq*t/fourpi2)+
     &        6.d0*htq**2*(t+xt*0.5d0+(1.5d0*htq+0.5d0*hbq-8.d0*g3)*
     &        (xt*t+t**2)/fourpi2)/fourpi2-3.d0*hbq**2*rmyouq**2
     &        *(1.d0+(9.d0*hbq-5.d0*htq-16.d0*g3)*t/fourpi2)/
     &        (96.d0*pi**2*rmsus2**2)
            rl3=0.25d0*(g2-g1)*(1.d0-3.d0*(htq+hbq)*t/fourpi2)+
     &        6.d0*htq*hbq*(t+atb*0.5d0+(htq+hbq-8.d0*g3)*(atb*t+t**2)
     &        /fourpi2)/fourpi2+3.d0*htq**2*(3.d0*rmyouq/rmsus2-
     &        rmyouq*at**2/(rmsus2**2))*(1.d0+(6.d0*htq-2.d0*hbq-16.d0*
     &        g3)*t/fourpi2)/(96.d0*pi**2)+3.d0*hbq**2*(3.d0*rmyouq/
     &        rmsus2-rmyouq*ab**2/(rmsus2**2))*(1.d0+(6.d0*hbq-2.d0*
     &        htq-16.d0*g3)*t/fourpi2)/(96.d0*pi**2)
            rl4=-0.5d0*g2*(1.d0-3.d0*(htq+hbq)*t/fourpi2)-
     &        6.d0*htq*hbq*(t+atb*0.5d0+(htq+hbq-8.d0*g3)*(atb*t+t**2)
     &        /fourpi2)/fourpi2+3.d0*htq**2*(3.d0*rmyouq/rmsus2-
     &        rmyouq*at**2/(rmsus2**2))*(1.d0+(6.d0*htq-2.d0*hbq-16.d0*
     &        g3)*t/fourpi2)/(96.d0*pi**2)+3.d0*hbq**2*(3.d0*rmyouq/
     &        rmsus2-rmyouq*ab**2/(rmsus2**2))*(1.d0+(6.d0*hbq-2.d0*
     &        htq-16.d0*g3)*t/fourpi2)/(96.d0*pi**2)
            rl5=-3.d0*htq**2*rmyouq*at**2*(1.d0-(2.d0*hbq-6.d0*htq+16.d0
     &        *g3)*t/fourpi2)/(96.d0*pi**2*rmsus2**2)-3.d0*hbq**2*rmyouq
     &        *ab**2*(1.d0-(2.d0*htq-6.d0*hbq+16.d0*g3)*t/fourpi2)/
     &        (96.d0*pi**2*rmsus2**2)
            rl6=3.d0*htq**2*rmyou**3*at*(1.d0-(3.5d0*hbq-7.5d0*htq+16.d0
     &        *g3)*t/fourpi2)/(96.d0*pi**2*rmsus2**2)+3.d0*hbq**2*rmyou*
     &        (ab**3/(rmsus**3)-6.d0*ab/rmsus)*(1.d0-(0.5d0*htq-4.
     &        5d0*hbq
     &        +16.d0*g3)*t/fourpi2)/(96.d0*pi**2*rmsus)
            rl7=3.d0*hbq**2*rmyou**3*ab*(1.d0-(3.5d0*htq-7.5d0*hbq+16.d0
     &        *g3)*t/fourpi2)/(96.d0*pi**2*rmsus2**2)+3.d0*htq**2*rmyou*
     &        (at**3/(rmsus**3)-6.d0*at/rmsus)*(1.d0-(0.5d0*hbq-4.
     &        5d0*htq
     &        +16.d0*g3)*t/fourpi2)/(96.d0*pi**2*rmsus)
* endel1
            rm12=v2*(sb*rcb*(rl3+rl4)+rl6*rcb2+rl7*sb2)-rma2*sb*rcb
            rm11=v2*(rl1*rcb2+2.d0*rl6*rcb*sb+rl5*sb2)+rma2*sb2
            rm22=v2*(rl2*sb2+2.d0*rl7*rcb*sb+rl5*rcb2)+rma2*rcb2
            trm=rm11+rm22
            detm=rm11*rm22-rm12**2
            rmh2=0.5d0*(trm-sqrt(trm**2-4.d0*detm))
          ENDIF !iloop
          rmh=sqrt(rmh2)
        ENDIF !irmhcomp
* determinazione di Dabelstein e del libro arancione
        tga=-tgb*(rma2+rmz2)/(rmz2+rma2*tgb**2-rmh2*(1.d0+tgb**2))
        IF(tga.LT.0.d0)THEN
          alf=atand(tga)
        ELSE
          alf=atand(tga)-180.d0
        ENDIF
        sa=sind(alf)
        rca=cosd(alf)
      ENDIF
** cr and cl couplings
** photon-electron left and right
      fel=-1.d0
      fer=-1.d0
** Zeta-electron left e right
      zer=(+1.d0*s2w)/rcw/sw
      zel=(-.5d0+1.d0*s2w)/rcw/sw
      zvr=0.d0
      zvl=.5d0/rcw/sw
** Zeta-quarkup left e right
      fqul=2.d0/3.d0
      fqur=fqul
      zqur=(-2.d0/3.d0*s2w)/rcw/sw
      zqul=(.5d0-2.d0/3.d0*s2w)/rcw/sw
** Zeta-quarkdown left e right
      fqdl=-1.d0/3.d0
      fqdr=fqdl
      zqdr=(1.d0/3.d0*s2w)/rcw/sw
      zqdl=(-.5d0+1.d0/3.d0*s2w)/rcw/sw
** W left (right=0)
      wcl=1.d0/sw/sqrt(2.d0)

* el1
** Higgs-ZZ * Higgs-bb coupling
      IF(iproc.GE.33.AND.isusy.EQ.0.AND.icch.NE.2)THEN
        IF(iproc.LE.39)THEN
          rhzz=rmb_run/(s2w*rc2w*2.d0)
** Higgs-WW * Higgs-bb coupling
          rhww=rmb_run/(s2w*2.d0)
          rhbb=rmb_run**2/(4.d0*rmw**2*s2w)
        ELSE IF(iproc.GT.39.AND.iproc.LE.46)THEN
          rhzz=rmc_run/(s2w*rc2w*2.d0)
** Higgs-WW * Higgs-bb coupling
          rhww=rmc_run/(s2w*2.d0)
          rhbb=rmc_run**2/(4.d0*rmw**2*s2w)
        ELSE IF(iproc.GT.46.AND.iproc.LE.53)THEN
          rhzz=rmtau/(s2w*rc2w*2.d0)
** Higgs-WW * Higgs-bb coupling
          rhww=rmtau/(s2w*2.d0)
          rhbb=rmtau**2/(4.d0*rmw**2*s2w)
        ENDIF
      ELSE IF(iproc.GE.33.AND.isusy.EQ.1.AND.icch.NE.2)THEN
        IF(iproc.LE.39)THEN
          rhzz=-rmb_run*sa*(sind(bet-alf))/(rcb*s2w*rc2w*2.d0)
          rhww=-rmb_run*sa*(sind(bet-alf))/(2.d0*s2w*rcb)
          rhbb=rmb_run**2*sa**2/(4.d0*rmw**2*s2w*rcb**2)
          IF(iproc.EQ.39.AND.isusy.EQ.1)THEN
            rzha=rmb_run**2*rmz*sa*sb*(cosd(bet-alf))/
     &           (8.d0*sw**3*rcb**2*rmw**3)
            rabb=-rmb_run**2*tgb**2/(4.d0*rmw**2*s2w)
          ENDIF
        ELSE IF(iproc.GT.39.AND.iproc.LE.46)THEN
          rhzz=rmc_run*rca*(sind(bet-alf))/(sb*s2w*rc2w*2.d0)
          rhww=rmc_run*rca*(sind(bet-alf))/(2.d0*s2w*sb)
          rhbb=rmc_run**2*rca**2/(4.d0*rmw**2*s2w*sb**2)
          IF(iproc.EQ.43.AND.isusy.EQ.1)THEN
            rzha=-rmc_run**2*rmz*rca*rcb*(cosd(bet-alf))/
     &           (8.d0*sw**3*sb**2*rmw**3)
            rabb=-rmc_run**2*rcb**2/(4.d0*rmw**2*s2w*sb**2)
          ENDIF
        ELSE IF(iproc.GT.46.AND.iproc.LE.53)THEN
          rhzz=-rmtau*sa*(sind(bet-alf))/(rcb*s2w*rc2w*2.d0)
          rhww=-rmtau*sa*(sind(bet-alf))/(2.d0*s2w*rcb)
          rhbb=rmtau**2*sa**2/(4.d0*rmw**2*s2w*rcb**2)
          IF(iproc.EQ.51.AND.isusy.EQ.1)THEN
            rzha=rmtau**2*rmz*sa*sb*(cosd(bet-alf))/
     &           (8.d0*sw**3*rcb**2*rmw**3)
            rabb=-rmtau**2*tgb**2/(4.d0*rmw**2*s2w)
          ENDIF
* fine
        ENDIF
      ENDIF

* endel1

      emcoupl=(1.d0/pi/alfainv)**4
      qcdcoupl=alfas_nc*alfainv
      qcdcor_nc=alfas_nc/pi
      qcdcor_cc=alfas_cc/pi
      qcdcor_h=alfas_h/pi*(5.67d0+42.74d0*alfas_h/pi)

      CALL initialize

      rmeisr=rme

* el1
      IF(imass.EQ.0.AND.iproc.LE.32)THEN
* endel1
        DO i=3,6
          xm(i)=0.d0
        ENDDO !i
        xmp=0.d0
        rme=0.d0
      ENDIF

      IF(igwcomp.EQ.1)THEN
        gamw=3.d0*rmw/(4.d0*s2w*alfainv)
        IF(i3q.EQ.1.OR.i5q.EQ.1)THEN
          gamw=gamw+istrcor*rmw/(2.d0*s2w*alfainv)*qcdcor_cc
        ELSE
          gamw=gamw+rmw/(2.d0*s2w*alfainv)*qcdcor_cc
        ENDIF
      ENDIF
      IF(igzcomp.EQ.1)THEN
* el1
        IF (rmb.EQ.0.d0.OR.(iproc.LE.32.AND.imass.EQ.0))THEN
* endel1
          gamz=21.d0-40.d0*s2w+160.d0*s2w**2/3.d0+(2.77d0/rmz)**4*
     &         (24.d0*s2w-16.d0*s2w**2)-9.d0*(2.77d0/rmz)**2
          IF(i3q.EQ.1.OR.i5q.EQ.1)THEN
            gamz=gamz+istrcor*qcdcor_nc*(15.d0-28.d0*s2w+88.d0*s2w**2/
     &           3.d0+9.d0*(2.77d0/rmz)**2*(2.d0*(-1.d0+4.d0*s2w/3.d0)
     &           **2-3.667d0)+(2.77d0/rmz)**4*(15.d0-33.d0*(-1.d0+4.d0*
     &           s2w/3.d0)**2))
          ELSE
            gamz=gamz+qcdcor_nc*(15.d0-28.d0*s2w+88.d0*s2w**2/3.d0+9.d0*
     &          (2.77d0/rmz)**2*(2.d0*(-1.d0+4.d0*s2w/3.d0)**2-3.667d0)+
     &          (2.77d0/rmz)**4*(15.d0-33.d0*(-1.d0+4.d0*s2w/3.d0)**2))
          ENDIF
          gamz=gamz*rmz/(24.d0*s2w*rc2w*alfainv)
* el1
        ELSE IF(rmb.NE.0.d0.AND.(iproc.GE.33.OR.(iproc.LE.32.AND.
     &          imass.EQ.1)))THEN
* endel1
          gamz=21.d0-40.d0*s2w+160.d0*s2w**2/3.d0+(rmb/rmz)**4*
     &         (24.d0*s2w-16.d0*s2w**2)-9.d0*(rmb/rmz)**2+istrcor*
     &         qcdcor_nc*(15.d0-28.d0*s2w+88.d0*s2w**2/3.d0+9.d0*
     &         (rmb/rmz)**2*(2.d0*(-1.d0+4.d0*s2w/3.d0)**2-3.667d0)+
     &         (rmb/rmz)**4*(15.d0-33.d0*(-1.d0+4.d0*s2w/3.d0)**2))
          gamz=gamz*rmz/(24.d0*s2w*rc2w*alfainv)
        ENDIF
      ENDIF
      gw=gamw/rmw
      gz=gamz/rmz

      IF(ighcomp.EQ.1.AND.iproc.GE.33.AND.icch.NE.2.AND.isusy.EQ.0)THEN
* el1
        IF (rmb.EQ.0.d0.AND.rmc.EQ.0.d0.AND.rmtau.EQ.0.d0) THEN
* endel1
          gamh=rmh*(3.d0*(rmb_run**2+rmc_run**2)*(1.d0+
     &         qcdcor_h*istrcor)+rmtau**2+(rmh*alfas_h/(3.d0*pi))**2)
     &         /(8.d0*rmw2*s2w*alfainv)
        ELSE
          gamh=rmh*(3.d0*(rmb_run**2*(1.d0-4.d0*(rmb/rmh)**2)**(1.5)+
     &         rmc_run**2*(1.d0-4.d0*(rmc/rmh)**2)**(1.5))*(1.d0+
     &         qcdcor_h*istrcor)+rmtau**2*(1.d0-4.d0*(rmtau/rmh)**2)**
     &         (1.5)+(rmh*alfas_h/(3.d0*pi))**2)
     &         /(8.d0*rmw2*s2w*alfainv)
        ENDIF
      ENDIF
      IF(ighcomp.EQ.1.AND.iproc.GE.33.AND.icch.NE.2.AND.isusy.EQ.1)THEN
* el1
        IF(rmb.EQ.0.d0.AND.rmc.EQ.0.d0.AND.rmtau.EQ.0.d0)THEN
* endel1
          gamh=(rmtau*sa/rcb)**2+3.d0*((rmb_run*sa/rcb)**2+
     &         (rmc_run*rca/sb)**2)*(1.d0+qcdcor_h*istrcor)
          gamh=gamh*rmh/(8.d0*rmw2*s2w*alfainv)
* el1
          IF(iproc.EQ.39.OR.iproc.EQ.43.OR.iproc.EQ.51)THEN
* endel1
            gama=(rmtau*sb/rcb)**2+3.d0*((rmb_run*sb/rcb)**2+
     &           (rmc_run*rcb/sb)**2)*(1.d0+qcdcor_h*istrcor)
            gama=gama*rma/(8.d0*rmw2*s2w*alfainv)
          ENDIF
        ELSE
          gamh=rmtau**2*(1.d0-(2.d0*rmtau/rmh)**2)**(1.5)*(sa/rcb)**2+
     &         3.d0*(rmb_run**2*(1.d0-(2.d0*rmb/rmh)**2)**(1.5)*
     &         (sa/rcb)**2+rmc_run**2*(1.d0-(2.d0*rmc/rmh)**2)**(1.5)*
     &         (rca/sb)**2)*(1.d0+qcdcor_h*istrcor)
          gamh=gamh*rmh/(8.d0*rmw2*s2w*alfainv)
* el1
          IF(iproc.EQ.39.OR.iproc.EQ.43.OR.iproc.EQ.51)THEN
* endel1
            gama=rmtau**2*sqrt(1.d0-(2.d0*rmtau/rma)**2)*(sb/rcb)**2+
     &           3.d0*(rmb_run**2*sqrt(1.d0-(2.d0*rmb/rma)**2)*
     &           (sb/rcb)**2+rmc_run**2*sqrt(1.d0-(2.d0*rmc/rma)**2)*
     &           (rcb/sb)**2)*(1.d0+qcdcor_h*istrcor)
            gama=gama*rma/(8.d0*rmw2*s2w*alfainv)
          ENDIF
        ENDIF
      ENDIF
      IF(iproc.GE.33.AND.icch.NE.2)THEN
        gh=gamh/rmh
* el1
        IF((iproc.EQ.39.OR.iproc.EQ.43.OR.iproc.EQ.51).AND.
     &     isusy.EQ.1)THEN
* endel1
          ga=gama/rma
        ENDIF
      ENDIF
      IF (icut.EQ.1) THEN
        DO i=3,6
          IF(icos.EQ.1)THEN
            beamcut_min(i)=thbeam_min(i)
            beamcut_max(i)=thbeam_max(i)
          ELSE IF(icos.EQ.0)THEN
            beamcut_min(i)=cos(thbeam_min(i)*pi/180.d0)
            beamcut_max(i)=cos(thbeam_max(i)*pi/180.d0)
          ENDIF
        ENDDO !i
        DO i=1,6
          IF(icos.EQ.1)THEN
            sepcut_min(i)=thsep_min(i)
            sepcut_max(i)=thsep_max(i)
          ELSE IF(icos.EQ.0)THEN
            sepcut_min(i)=cos(thsep_min(i)*pi/180.d0)
            sepcut_max(i)=cos(thsep_max(i)*pi/180.d0)
          ENDIF
          rm_min2(i)=rm_min(i)**2
          rm_max2(i)=rm_max(i)**2
        ENDDO !i
      ENDIF
      IF(ianc.EQ.1)THEN
        yf=yf/rmw2
        yz=yz/rmw2
        zz=zz/rmw2
      ENDIF

* Estremi di integrazione sulle masse m34 ed m56:
      IF(icut.EQ.0.OR.rm_min(1).EQ.0.d0)THEN
        estrinf_34=xm(3)+xm(4)
      ELSE IF(icut.EQ.1.AND.rm_min(1).NE.0.d0)THEN
        estrinf_34=rm_min(1)
      ENDIF
      IF(icut.EQ.0.OR.rm_min(3).EQ.0.d0)THEN
        estrinf_36=xm(3)+xm(6)
      ELSE IF(icut.EQ.1.AND.rm_min(3).NE.0.d0)THEN
        estrinf_36=rm_min(3)
      ENDIF
      IF(icut.EQ.0.OR.rm_min(6).EQ.0.d0)THEN
        estrinf_56=xm(5)+xm(6)
      ELSE IF(icut.EQ.1.AND.rm_min(6).NE.0.d0)THEN
        estrinf_56=rm_min(6)
      ENDIF
      IF(icut.EQ.0.OR.rm_min(4).EQ.0.d0)THEN
        estrinf_54=xm(4)+xm(5)
      ELSE IF(icut.EQ.1.AND.rm_min(4).NE.0.d0)THEN
        estrinf_54=rm_min(4)
      ENDIF

*195
      if(ismallangle.gt.0) then
        estrinf_456=max(xm(4)+xm(5)+xm(6),xm(4)+rm_min(6),
     &    xm(5)+rm_min(5),xm(6)+rm_min(4))
      endif
*195end

* el1
      IF((iproc.EQ.39.OR.iproc.EQ.43.OR.iproc.EQ.51).AND.
     &    icch.NE.2.AND.isusy.EQ.1)THEN
* endel1
        estrmed2=0.5d0*(rma+rmz)
      ENDIF
      cim=(0.d0,1.d0)
      IF(isr.EQ.1)THEN
        rl=log(s_col/rmeisr/rmeisr)
        alfa_me=1.d0/137.0359895d0
        beta=2.d0*alfa_me*(rl-1.d0)/pi
        gamma=exp(gammln(1.d0+beta/2.d0))
        ge=0.5772156649d0
        rlim=exp(beta*(0.75d0-ge)/2.d0)*(beta/2.d0)/gamma
      ENDIF
      IF(ipr.EQ.0)THEN
        czipr=(1.d0,0.d0)
        ccz=cim*gamz*rmz
        cwipr=(1.d0,0.d0)
        ccw=cim*gamw*rmw
        IF(iproc.GE.33.AND.icch.NE.2)THEN
          chipr=(1.d0,0.d0)
          cch=cim*gamh*rmh
* el1
          IF((iproc.EQ.39.OR.iproc.EQ.43.OR.iproc.EQ.51).
     &       and.isusy.EQ.1)THEN
* endel1
            caipr=(1.d0,0.d0)
            cca=cim*gama*rma
          ENDIF
        ENDIF
      ELSE IF(ipr.EQ.1)THEN
        czipr=1.d0+cim*gamz/rmz
        ccz=(0.d0,0.d0)
        cwipr=1.d0+cim*gamw/rmw
        ccw=(0.d0,0.d0)
        IF(iproc.GE.33.AND.icch.NE.2)THEN
          chipr=1.d0+cim*gamh/rmh
          cch=(0.d0,0.d0)
* el1
          IF((iproc.EQ.39.OR.iproc.EQ.43.OR.iproc.EQ.51).AND.
     &       isusy.EQ.1)THEN
* endel1
            caipr=1.d0+cim*gama/rma
            cca=(0.d0,0.d0)
          ENDIF
        ENDIF
      ELSE IF(ipr.EQ.2)THEN
        czipr=(1.d0,0.d0)
        ccz=cim*gamz*rmz
        cwipr=1.d0+cim*gamw/rmw
        ccw=(0.d0,0.d0)
        IF(iproc.GE.33.AND.icch.NE.2)THEN
          chipr=1.d0+cim*gamh/rmh
          cch=(0.d0,0.d0)
* el1
          IF((iproc.EQ.39.OR.iproc.EQ.43.OR.iproc.EQ.51).AND.
     &      isusy.EQ.1)THEN
* endel1
            caipr=1.d0+cim*gama/rma
            cca=(0.d0,0.d0)
          ENDIF
        ENDIF
      ENDIF


* begin EZ: test di compatibilita' fra ipr e ifloop
      IF((ifloop.EQ.1).AND.(ipr.NE.2)) THEN
        PRINT*,'ERROR:'
        PRINT*,'ifloop.EQ.1  and ipr.NE.2'
        STOP
      ENDIF
      IF((ifloop.NE.1).AND.(ipr.EQ.2)) THEN
        PRINT*,'ERROR:'
        PRINT*,'ipr.EQ.2 and ifloop.NE.1'
        STOP
      ENDIF
* end EZ

      CALL printer(rmh,gamh,rma,gama,tgb,rmb)

* el1
* ridefinisco la massa per i nuovi processi con Higgs
      IF(iproc.GE.40.AND.iproc.LE.46)THEN
        rmb=rmc
        rmb2=rmc**2
        rmb_run=rmc_run
        rmt2=0.d0   !assuming the s quark to be massless
      ELSE IF(iproc.GE.47.AND.iproc.LE.53)THEN
        rmb=rmtau
        rmb2=rmtau**2
        rmb_run=rmtau
        rmt2=0.d0
      ENDIF
* endel1

* Routine Vegas parameter:
* sa1
Cqedps
C      IF(isr.EQ.0)THEN
C        ndim=7
C      ELSE
C        ndim=9
C      ENDIF
      IF(isr.EQ.0.OR.isr.EQ.2)THEN
        ndim=7
      ELSEIF (isr.EQ.1) THEN
        ndim=9
      ELSE
        PRINT*, 'ERROR: isr value not admitted'
        STOP
      ENDIF
Cqedpsend
* sa1end

* Integration limits:
      DO i=1,ndim
        region(i)=0.d0
        region(ndim+i)=1.d0
      ENDDO !i
* Thermalization:
      IF(iterm.EQ.1.AND.irepeat.EQ.0)THEN
        init=0
        PRINT*,' '
        PRINT*,'Thermalization'
* CC thermalization
        IF (iccnc.EQ.1.OR.iccnc.EQ.3) THEN
*agg
          IF(iccnc.EQ.3.AND.interf.EQ.1)THEN
            imix=2
            icc=-1
          ELSE IF(iccnc.EQ.1.OR.(iccnc.EQ.3.AND.interf.EQ.0))THEN
            imix=1
            icc=1
          ENDIF
*aggend
          ips=ips_cc
          rmx1=rmw
          gamx1=gamw
          gx1=gw
          rmx2=rmx1
          gamx2=gamx1
          gx2=gx1
* Higgs thermalization
* el1
        ELSE IF(iccnc.EQ.4.AND.(icch.EQ.1.OR.icch.EQ.3))THEN
          IF(isusy.EQ.0.OR.(isusy.EQ.1.AND.(iproc.NE.39.AND.iproc.
     &      ne.43.AND.iproc.NE.51)).OR.(isusy.EQ.1.AND.
     &      (iproc.EQ.39.OR.iproc.EQ.43.OR.iproc.EQ.51).AND.
     &      ((rma+rmh).gt.e_cm.or.iha.eq.2.or.iha.eq.4)))then
* endel1
            imix=1
            ips=ips_cc
            rmx1=rmh
            gamx1=gamh
            gx1=gh
            rmx2=rmz
            gamx2=gamz
            gx2=gz
* el1
          ELSE IF(isusy.EQ.1.AND.(iproc.EQ.39.OR.iproc.EQ.43.OR.
     &            iproc.EQ.51).AND.(rma+rmh).le.e_cm.and.iha.ne.
     &            2.AND.iha.NE.4)THEN
* endel1
* hA + hZ thermalization
            imix=1
            ips=ips_cc
            rmx1=rmh
            gamx1=gamh
            gx1=gh
            rmx2=min(rma,rmz)
            IF(rma.LT.rmz)THEN
              gamx2=gama
              gx2=ga
            ELSE
              gamx2=gamz
              gx2=gz
            ENDIF
            rmx3=max(rma,rmz)
            IF(rma.GT.rmz)THEN
              gamx3=gama
              gx3=ga
            ELSE
              gamx3=gamz
              gx3=gz
            ENDIF
          ENDIF
* NC thermalization
        ELSE
          imix=-1
          icc=0
          ips=ips_nc
          rmx1=rmz
          gamx1=gamz
          gx1=gz
          rmx2=rmz
          gamx2=gamz
          gx2=gz
        ENDIF
* estremi di integrazione sulle due masse invarianti
        IF(iproc.GE.6.AND.iproc.LE.8.AND.(iccnc.EQ.1.OR.iccnc.EQ.3))THEN
          estrinf1=estrinf_36
          estrinf2=estrinf_54
*195    ELSE IF((iproc.EQ.4.OR.iproc.EQ.2.OR.iproc.EQ.7.OR.iproc.EQ.21)
*195     &    .AND.ismallangle.EQ.1)THEN
*195          estrinf1=xm(4)+xm(5)+xm(6)
*195          estrinf2=xm(5)+xm(4)
        ELSE
          estrinf1=estrinf_34
          estrinf2=estrinf_56
        ENDIF
        smin=(estrinf1+estrinf2)**2
*195
        if (ismallangle.gt.0) then
          estrinf1=estrinf_456
        endif
*195end
        IF(ipr.EQ.0)THEN
          x1_min=atan((estrinf1**2-rmx1**2)/(gamx1*rmx1))
          x2_min=atan((estrinf2**2-rmx2**2)/(gamx2*rmx2))
        ELSE
          x1_min=atan(((1.d0+gx1**2)*estrinf1**2-rmx1**2)/(gx1*rmx1**2))
          x2_min=atan(((1.d0+gx2**2)*estrinf2**2-rmx2**2)/(gx2*rmx2**2))
        ENDIF
        IF(ism34.NE.-1) then
          IF(imix.LT.0)THEN
            ism34=1
          ELSE
            ism34=0
          ENDIF
        ENDIF
        IF(ism56.NE.-1) then
          IF(imix.LT.0)THEN
            ism56=1
          ELSE
            ism56=0
          ENDIF
        ENDIF
        IF(iproc.EQ.7)THEN
          IF(imix.GT.0)THEN
            rexp5=0.0d0
            rexp6=0.7d0
          ELSE
            rexp5=0.0d0
            rexp6=0.0d0
          ENDIF
        ENDIF
        CALL vegas(region,ndim,fxn,init,ncall_term,itmx_term,nprn,
     &               avgi1,sd1,rchi2a,acc,y1,it1,ndo1,si1,swgt1,schi1)
        IF(iccnc.EQ.3.OR.(iccnc.EQ.4.AND.icch.EQ.3))THEN
          iseed1=idum
          iy1=iy
          idum21=idum2
          DO n=1,32
            iv1(n)=iv(n)
          ENDDO !n
          init=0
          idum=0
          idum2=123456789
          iy=0
          DO n=1,32
            iv(n)=0
          ENDDO !n
*agg
          IF(iccnc.EQ.3.AND.interf.EQ.1)THEN
            imix=-1
            icc=0
          ELSE IF((iccnc.EQ.3.AND.interf.EQ.0).OR.
     &           (iccnc.EQ.4.AND.icch.EQ.3))THEN
            imix=-2
            icc=-1
          ENDIF
*aggend
          ips=ips_nc
          rmx1=rmz
          gamx1=gamz
          gx1=gz
          rmx2=rmz
          gamx2=gamz
          gx2=gz
          estrinf1=estrinf_34
          estrinf2=estrinf_56
          smin=(estrinf1+estrinf2)**2
*195
          if (ismallangle.gt.0) then
            estrinf1=estrinf_456
          endif
*195end
          IF(ipr.EQ.0)THEN
            x1_min=atan((estrinf1**2-rmx1**2)/(gamx1*rmx1))
            x2_min=atan((estrinf2**2-rmx2**2)/(gamx2*rmx2))
          ELSE
            x1_min=atan(((1.d0+gx1**2)*estrinf1**2-rmx1**2)
     &                 /(gx1*rmx1**2))
            x2_min=atan(((1.d0+gx2**2)*estrinf2**2-rmx2**2)
     &                 /(gx2*rmx2**2))
          ENDIF
          IF(ism34.NE.-1) then
            IF(imix.LT.0)THEN
              ism34=1
            ELSE
              ism34=0
            ENDIF
          ENDIF
          IF(ism56.NE.-1) then
            IF(imix.LT.0)THEN
              ism56=1
            ELSE
              ism56=0
            ENDIF
          ENDIF
          IF(iproc.EQ.7)THEN
            IF(imix.GT.0)THEN
              rexp5=0.0d0
              rexp6=0.7d0
            ELSE
              rexp5=0.0d0
              rexp6=0.0d0
            ENDIF
          ENDIF
          CALL vegas(region,ndim,fxn,init,ncall_term,itmx_term,nprn,
     &                 avgi2,sd2,rchi2a,acc,y2,it2,ndo2,si2,swgt2,schi2)
          iseed2=idum
          iy2=iy
          idum22=idum2
          DO n=1,32
            iv2(n)=iv(n)
          ENDDO !n
        ENDIF
      ENDIF !thermalization
*agg
* initialization of the variables used if IFLAT=1
      IF(iflat.EQ.1)THEN
* n. random generato con RAN(ISEED)
        iseed=19753179
* numero di punti che eccedono il massimo fissato
        novermax=0
* n. di momenti generati
        nevent=0
* maximum function value at first and second vegas iteration
        rmaxfxn=0.d0
        rmaxfxn_1it=0.d0
        rmaxfxn_2it=0.d0
        rmaxfxn_cc_1it=0.d0
        rmaxfxn_nc_1it=0.d0
        rmaxfxn_cc_2it=0.d0
        rmaxfxn_nc_2it=0.d0
        IF(istormom.EQ.1)THEN
          IF(iccnc.EQ.3.OR.(iccnc.EQ.4.AND.icch.EQ.3))THEN
            OPEN(unit=21,file='ABMOM_SIGN.DAT',status='new')
            OPEN(unit=23,file='ABMOM_BACK.DAT',status='new')
          ELSE
            OPEN(unit=23,file='ABMOM.DAT',status='new')
          ENDIF
        ENDIF
      ENDIF
*aggend
* inizializzazione di variabili usate se  IDISTR=1
      IF(idistr.EQ.1)THEN
        DO m=1,ndistr
          test(m)=0.d0
          IF(iccnc.EQ.3.OR.(iccnc.EQ.4.AND.icch.EQ.3))THEN
            test_mix(m)=0.d0
          ENDIF
          DO i=1,nbinmax
            DO j=1,nitmax
              distr_local(m,i,j)=0.d0
              dev_local(m,i,j)=0.d0
              ncallbin(m,i,j)=0
              IF(iccnc.EQ.3.OR.(iccnc.EQ.4.AND.icch.EQ.3))THEN
                distr_loc_mix(m,i,j)=0.d0
                dev_loc_mix(m,i,j)=0.d0
                ncallbin_mix(m,i,j)=0
              ENDIF
            ENDDO !j
          ENDDO !i
        ENDDO !m
        DO m=1,ndistr
          DO i=1,nitmax
            tail_local(m,i)=0.d0
            IF(iccnc.EQ.3.OR.(iccnc.EQ.4.AND.icch.EQ.3))THEN
              tail_loc_mix(m,i)=0.d0
            ENDIF
          ENDDO !i
        ENDDO !m
        DO m=1,ndistr
          DO i=1,nintmax
            nbin_sum(m,i)=0
          ENDDO !i
        ENDDO !m
        DO m=1,ndistr
          DO i=1,nsubint(m)
            bin_width(m,i)=(distr_estrinf(m,i+1)-distr_estrinf(m,i))/
     &        nbin_number(m,i)
          ENDDO !i
        ENDDO !m
      ENDIF
* integration loop
      IF(iterm.EQ.1)THEN
        init=1
      ELSE
        init=0
      ENDIF
      IF(iccnc.EQ.1.OR.iccnc.EQ.2.OR.(iccnc.EQ.4.AND.icch.NE.3))THEN
        IF(iccnc.EQ.1)THEN
          PRINT*,' '
          PRINT*,'CC process'
          imix=1
          icc=1
          ips=ips_cc
          rmx1=rmw
          gamx1=gamw
          gx1=gw
          rmx2=rmw
          gamx2=gamw
          gx2=gw
        ELSE IF(iccnc.EQ.2.OR.(iccnc.EQ.4.AND.icch.EQ.2))THEN
          IF (iccnc.EQ.2) THEN
            PRINT*,' '
            PRINT*,'NC process'
          ELSE
            PRINT*,' '
            PRINT*,'Higgs background'
          ENDIF
          imix=-1
          icc=0
          ips=ips_nc
          rmx1=rmz
          gamx1=gamz
          gx1=gz
          rmx2=rmz
          gamx2=gamz
          gx2=gz
        ELSE IF (iccnc.EQ.4.AND.icch.EQ.1) THEN
* el1
          IF(isusy.EQ.0.OR.(isusy.EQ.1.AND.iproc.NE.39.AND.
     &      iproc.NE.43.AND.iproc.NE.51).OR.(isusy.EQ.1.AND.
     &      (iproc.EQ.39.OR.iproc.EQ.43.OR.iproc.EQ.51).AND.
     &      ((rma+rmh).gt.e_cm.or.iha.eq.2.or.iha.eq.4)))then
            PRINT*,' '
            IF(iproc.NE.39.AND.iproc.NE.43.AND.iproc.NE.51)THEN
              PRINT*,'Higgs signal'
            ELSE IF((iproc.EQ.39.OR.iproc.EQ.43.OR.iproc.EQ.51).
     &               and.isusy.EQ.0.AND.iha.EQ.1)THEN
              PRINT*,'Complete h contribution'
            ELSE IF((iproc.EQ.39.OR.iproc.EQ.43.OR.iproc.EQ.51).
     &               and.isusy.EQ.0.AND.iha.EQ.2)THEN
              PRINT*,'only hZ contribution'
            ELSE IF((iproc.EQ.39.OR.iproc.EQ.43.OR.iproc.EQ.51).
     &               and.isusy.EQ.1.AND.iha.EQ.1)THEN
              PRINT*,'Complete h and A higgs contribution'
            ELSE IF((iproc.EQ.39.OR.iproc.EQ.43.OR.iproc.EQ.51).
     &               and.isusy.EQ.1.AND.iha.EQ.2)THEN
              PRINT*,'Complete h contribution'
            ELSE IF((iproc.EQ.39.OR.iproc.EQ.43.OR.iproc.EQ.51).
     &               and.isusy.EQ.1.AND.iha.EQ.3)THEN
              PRINT*,'Complete A contribution'
            ELSE IF((iproc.EQ.39.OR.iproc.EQ.43.OR.iproc.EQ.51).
     &               and.isusy.EQ.1.AND.iha.EQ.4)THEN
              PRINT*,'only hZ contribution'
            ELSE IF((iproc.EQ.39.OR.iproc.EQ.43.OR.iproc.EQ.51).
     &               and.isusy.EQ.1.AND.iha.EQ.5)THEN
              PRINT*,'Only hA contribution'
            ELSE IF((iproc.EQ.39.OR.iproc.EQ.43.OR.iproc.EQ.51).
     &               and.isusy.EQ.1.AND.iha.EQ.6)THEN
              PRINT*,'Only hZ+hA contribution'
            ENDIF
* endel1
            imix=1
            ips=ips_cc
            rmx1=rmh
            gamx1=gamh
            gx1=gh
            rmx2=rmz
            gamx2=gamz
            gx2=gz
* el1
          ELSE IF(isusy.EQ.1.AND.(iproc.EQ.39.OR.iproc.EQ.43.OR.
     &            iproc.EQ.51).AND.(rma+rmh).le.e_cm.and.
     &            iha.NE.2.AND.iha.NE.4)THEN
            PRINT*,' '
            IF((iproc.EQ.39.OR.iproc.EQ.43.OR.iproc.EQ.51).
     &          and.iha.EQ.1)THEN
              PRINT*,'Complete h and A higgs contribution'
            ELSE IF((iproc.EQ.39.OR.iproc.EQ.43.OR.iproc.EQ.51).
     &               and.iha.EQ.3)THEN
              PRINT*,'Complete A contribution'
            ELSE IF((iproc.EQ.39.OR.iproc.EQ.43.OR.iproc.EQ.51).
     &               and.iha.EQ.5)THEN
              PRINT*,'only hA contribution'
            ELSE IF((iproc.EQ.39.OR.iproc.EQ.43.OR.iproc.EQ.51).
     &               and.iha.EQ.6)THEN
              PRINT*,'only hZ+hA contribution'
            ENDIF
* endel1
            imix=1
            ips=ips_cc
            rmx1=rmh
            gamx1=gamh
            gx1=gh
            rmx2=min(rma,rmz)
            IF(rma.LT.rmz)THEN
              gamx2=gama
              gx2=ga
            ELSE
              gamx2=gamz
              gx2=gz
            ENDIF
            rmx3=max(rma,rmz)
            IF(rma.GT.rmz)THEN
              gamx3=gama
              gx3=ga
            ELSE
              gamx3=gamz
              gx3=gz
            ENDIF
          ENDIF
        ENDIF
        IF(iproc.GE.6.AND.iproc.LE.8.AND.iccnc.EQ.1)THEN
          estrinf1=estrinf_36
          estrinf2=estrinf_54
        ELSE
          estrinf1=estrinf_34
          estrinf2=estrinf_56
        ENDIF
        smin=(estrinf1+estrinf2)**2
*195
        if (ismallangle.gt.0) then
          estrinf1=estrinf_456
        endif
*195end
        IF(ipr.EQ.0)THEN
          x1_min=atan((estrinf1**2-rmx1**2)/(gamx1*rmx1))
          x2_min=atan((estrinf2**2-rmx2**2)/(gamx2*rmx2))
        ELSE
          x1_min=atan(((1.d0+gx1**2)*estrinf1**2-rmx1**2)/(gx1*rmx1**2))
          x2_min=atan(((1.d0+gx2**2)*estrinf2**2-rmx2**2)/(gx2*rmx2**2))
        ENDIF
*agg
        IF(iflat.EQ.0)THEN
          IF(ism34.NE.-1) then
            IF(imix.LT.0)THEN
              ism34=1
            ELSE
              ism34=0
            ENDIF
          ENDIF
          IF(ism56.NE.-1) then
            IF(imix.LT.0)THEN
              ism56=1
            ELSE
              ism56=0
            ENDIF
          ENDIF
          IF(iproc.EQ.7)THEN
            IF(imix.GT.0)THEN
              rexp5=0.0d0
              rexp6=0.7d0
            ELSE
              rexp5=0.0d0
              rexp6=0.0d0
            ENDIF
          ENDIF
          CALL vegas(region,ndim,fxn,init,ncall,itmx,nprn,avgi1,
     &                 sd1,rchi2a,acc,y1,it1,ndo1,si1,swgt1,schi1)
        ELSE IF(iflat.EQ.1)THEN
          acc=0.d0
          DO nit=1,2
            IF(nit.EQ.2)init=2
            IF((nit.EQ.1.AND.irepeat.EQ.0).OR.(nit.EQ.2))THEN
              IF(irepeat.GE.1)THEN
                it1=2
                OPEN(unit=24,file='abvegas.dat',status='old')
                READ(24,*)idum,idum2,iy
                DO i=1,32
                  READ(24,*)iv(i)
                ENDDO !i
                READ(24,*)avgi1,sd1,rchi2a
                DO i=1,50
                  DO j=1,10
                    READ(24,*)y1(i,j)
                  ENDDO !j
                ENDDO !i
                READ(24,*)ndo1,si1,swgt1,schi1
                READ(24,*)rmaxfxn
                READ(24,*),avgi_tot,sd_tot
                CLOSE(24)
C26
                IF(irepeat.GT.1) nprn=-1
C26
              ENDIF
              IF(ism34.NE.-1) then
                IF(imix.LT.0)THEN
                  ism34=1
                ELSE
                  ism34=0
                ENDIF
              ENDIF
              IF(ism56.NE.-1) then
                IF(imix.LT.0)THEN
                  ism56=1
                ELSE
                  ism56=0
                ENDIF
              ENDIF
              IF(iproc.EQ.7)THEN
                IF(imix.GT.0)THEN
                  rexp5=0.0d0
                  rexp6=0.7d0
                ELSE
                  rexp5=0.0d0
                  rexp6=0.0d0
                ENDIF
              ENDIF
              CALL vegas(region,ndim,fxn,init,ncall,nit,nprn,avgi1,
     &                     sd1,rchi2a,acc,y1,it1,ndo1,si1,swgt1,schi1)
              IF(nit.EQ.1)THEN
                rmaxfxn_1it=rmaxfxn
              ENDIF
              IF(istorvegas.EQ.1.AND.nit.EQ.1)THEN
                OPEN(unit=24,file='abvegas.dat',status='new')
                WRITE(24,*)idum,idum2,iy
                DO i=1,32
                  WRITE(24,*)iv(i)
                ENDDO !i
                WRITE(24,*)avgi1,sd1,rchi2a
                DO i=1,50
                  DO j=1,10
                    WRITE(24,*)y1(i,j)
                  ENDDO !j
                ENDDO !i
                WRITE(24,*)ndo1,si1,swgt1,schi1
              ENDIF
              IF(istorvegas.EQ.1.AND.nit.EQ.2)THEN
                rmaxfxn=rmaxfxn_2it
                WRITE(24,*)rmaxfxn
                WRITE(24,*)avgi1,sd1
                CLOSE(24)
              ENDIF
            ENDIF
          ENDDO !nit
        ENDIF
*aggend
        IF (irepeat.NE.2) THEN
          avgi_tot=avgi1
          sd_tot=sd1
        ENDIF
        it=it1-1
      ELSE IF(iccnc.EQ.3.OR.(iccnc.EQ.4.AND.icch.EQ.3))THEN
        IF(iflat.EQ.1)acc=0.d0
        DO nit=1,itmx
          IF(nit.GE.2)init=2
          ncont_cc=0
          DO irip=1,2
            IF((irip.EQ.1.AND.((nit.EQ.1.AND.irepeat.EQ.0).OR.
     &        (iflat.EQ.1.AND.nit.EQ.2).OR.(sd_tot.GT.acc*avgi_tot.and.
     &        sd1.GT.r*acc*avgi_tot.and.sd1*avgi2.gt.sd2*avgi1.and.
     &        irepeat.EQ.0))).OR.(irip.EQ.2.AND.ncont_cc.EQ.0.AND.
     &        sd_tot.GT.acc*avgi_tot.and.sd1.gt.r*acc*avgi_tot))then
              ncont_cc=1
              idum=iseed1
              iy=iy1
              idum2=idum21
              DO n=1,32
                iv(n)=iv1(n)
              ENDDO !n
*agg
              IF(iccnc.EQ.3.AND.interf.EQ.1)THEN
                imix=2
                icc=-1
              ELSE IF((iccnc.EQ.3.AND.interf.EQ.0).OR.
     &               (iccnc.EQ.4.AND.icch.EQ.3))THEN
                imix=1
                icc=1
              ENDIF
*aggend
              ips=ips_cc
              IF(iccnc.EQ.3)THEN
                PRINT*,' '
*agg
                IF(interf.EQ.0)THEN
                  PRINT*,'CC process'
                ELSE
                  PRINT*,'CC process + CC-NC interference'
                ENDIF
*aggend
                rmx1=rmw
                gamx1=gamw
                gx1=gw
                rmx2=rmw
                gamx2=gamw
                gx2=gw
              ELSE
* el1
                IF(isusy.EQ.0.OR.(isusy.EQ.1.AND.(iproc.NE.39.
     &            and.iproc.NE.43.AND.iproc.NE.51)).OR.(isusy.
     &            eq.1.AND.(iproc.EQ.39.OR.iproc.EQ.43.OR.iproc.EQ.51).
     &            and.(rma+rmh).gt.e_cm))then
                  PRINT*,' '
                  IF(iproc.NE.39.AND.iproc.NE.43.AND.iproc.NE.51)THEN
                    PRINT*,'Higgs signal'
                  ELSE IF((iproc.EQ.39.OR.iproc.EQ.43.OR.iproc.EQ.51).
     &                     and.isusy.EQ.0)THEN
                    PRINT*,'Complete higgs contribution'
                  ELSE IF((iproc.EQ.39.OR.iproc.EQ.43.OR.iproc.EQ.51).
     &                     and.isusy.EQ.1)THEN
                    PRINT*,'Complete h and A higgs contribution'
                  ENDIF
* endel1
                  rmx1=rmh
                  gamx1=gamh
                  gx1=gh
                  rmx2=rmz
                  gamx2=gamz
                  gx2=gz
* el1
                ELSE IF(isusy.EQ.1.AND.(iproc.EQ.39.OR.iproc.EQ.43.OR.
     &                  iproc.EQ.51).AND.(rma+rmh).le.e_cm)then
* endel1
                  PRINT*,' '
                  PRINT*,'Complete h and A higgs contribution'
                  rmx1=rmh
                  gamx1=gamh
                  gx1=gh
                  rmx2=min(rma,rmz)
                  IF(rma.LT.rmz)THEN
                    gamx2=gama
                    gx2=ga
                  ELSE
                    gamx2=gamz
                    gx2=gz
                  ENDIF
                  rmx3=max(rma,rmz)
                  IF(rma.GT.rmz)THEN
                    gamx3=gama
                    gx3=ga
                  ELSE
                    gamx3=gamz
                    gx3=gz
                  ENDIF
                ENDIF
              ENDIF
              IF(iccnc.EQ.3)THEN
                estrinf1=estrinf_36
                estrinf2=estrinf_54
              ELSE
                estrinf1=estrinf_34
                estrinf2=estrinf_56
              ENDIF
              smin=(estrinf1+estrinf2)**2
*195
              if (ismallangle.gt.0) then
                estrinf1=estrinf_456
              endif
*195end
              IF(ipr.EQ.0)THEN
                x1_min=atan((estrinf1**2-rmx1**2)/(gamx1*rmx1))
                x2_min=atan((estrinf2**2-rmx2**2)/(gamx2*rmx2))
              ELSE
                x1_min=atan(((1.d0+gx1**2)*estrinf1**2-rmx1**2)
     &                     /(gx1*rmx1**2))
                x2_min=atan(((1.d0+gx2**2)*estrinf2**2-rmx2**2)
     &                     /(gx2*rmx2**2))
              ENDIF
*agg
              IF(nit.EQ.2.AND.irepeat.GE.1)THEN
                it1=2
                OPEN(unit=24,file='abvegas_cc.dat',status='old')
                READ(24,*)idum,idum2,iy
                DO i=1,32
                  READ(24,*)iv(i)
                ENDDO !i
                READ(24,*)avgi1,sd1,rchi2a
                DO i=1,50
                  DO j=1,10
                    READ(24,*)y1(i,j)
                  ENDDO !j
                ENDDO !i
                READ(24,*)ndo1,si1,swgt1,schi1
                READ(24,*)rmaxfxn
                IF(irepeat.EQ.2)THEN
                  READ(24,*)sigma1,sdfin1
                  READ(24,*)sigma2,sdfin2
                  nflevts1=nflevts*sigma1/(sigma1+sigma2)
                  nflevtstot=nflevts
                  nflevts=nflevts1
                ENDIF
                CLOSE(24)
C26
                IF(irepeat.GT.1) nprn=-1
C26
              ENDIF
*aggend
              IF(ism34.NE.-1) then
                IF(imix.LT.0)THEN
                  ism34=1
                ELSE
                  ism34=0
                ENDIF
              ENDIF
              IF(ism56.NE.-1) then
                IF(imix.LT.0)THEN
                  ism56=1
                ELSE
                  ism56=0
                ENDIF
              ENDIF
              IF(iproc.EQ.7)THEN
                IF(imix.GT.0)THEN
                  rexp5=0.0d0
                  rexp6=0.7d0
                ELSE
                  rexp5=0.0d0
                  rexp6=0.0d0
                ENDIF
              ENDIF
              CALL vegas(region,ndim,fxn,init,ncall,nit,nprn,avgi1,
     &                     sd1,rchi2a,acc,y1,it1,ndo1,si1,swgt1,schi1)
              IF (irepeat.GE.1)  PRINT 205,nevent

*agg
              IF(nit.EQ.1.AND.istorvegas.EQ.1)THEN
                OPEN(unit=24,file='abvegas_cc.dat',status='new')
                WRITE(24,*)idum,idum2,iy
                DO i=1,32
                  WRITE(24,*)iv(i)
                ENDDO !i
                WRITE(24,*)avgi1,sd1,rchi2a
                DO i=1,50
                  DO j=1,10
                    WRITE(24,*)y1(i,j)
                  ENDDO !j
                ENDDO !i
                WRITE(24,*)ndo1,si1,swgt1,schi1
              ENDIF
              IF(nit.EQ.2.AND.istorvegas.EQ.1)THEN
                sigma1=avgi1
              ENDIF
*aggend
              resl_cc(it1-1)=resl(it1-1)
              iseed1=idum
              iy1=iy
              idum21=idum2
              DO n=1,32
                iv1(n)=iv(n)
              ENDDO !n
              avgi_tot=avgi1+avgi2
              sd_tot=sqrt(sd1**2+sd2**2)
            ENDIF
            IF(irip.EQ.1.AND.((nit.EQ.1.AND.irepeat.EQ.0).OR.
     &        (iflat.EQ.1.AND.nit.EQ.2).OR.(sd_tot.GT.acc*avgi_tot.and.
     &        sd2.GT.r*acc*avgi_tot.and.irepeat.eq.0)))then
              idum=iseed2
              iy=iy2
              idum2=idum22
              DO n=1,32
                iv(n)=iv2(n)
              ENDDO !n
              IF(iccnc.EQ.3.AND.interf.EQ.1)THEN
                imix=-1
                icc=0
              ELSE IF((iccnc.EQ.3.AND.interf.EQ.0).OR.
     &               (iccnc.EQ.4.AND.icch.EQ.3))THEN
                imix=-2
                icc=-1
              ENDIF
              ips=ips_nc
              IF (iccnc.EQ.3) THEN
                PRINT*,' '
*agg
                IF(interf.EQ.0)THEN
                  PRINT*,'NC process + CC-NC interference'
                ELSE
                  PRINT*,'NC process'
                ENDIF
*aggend
              ELSE
                PRINT*,' '
                PRINT*,'Higgs backg + Higgs sign-backg intereference'
              ENDIF
              rmx1=rmz
              gamx1=gamz
              gx1=gz
              rmx2=rmz
              gamx2=gamz
              gx2=gz
              estrinf1=estrinf_34
              estrinf2=estrinf_56
              smin=(estrinf1+estrinf2)**2
*195
              if (ismallangle.gt.0) then
                estrinf1=estrinf_456
              endif
*195end
              IF(ipr.EQ.0)THEN
                x1_min=atan((estrinf1**2-rmx1**2)/(gamx1*rmx1))
                x2_min=atan((estrinf2**2-rmx2**2)/(gamx2*rmx2))
              ELSE
                x1_min=atan(((1.d0+gx1**2)*estrinf1**2-rmx1**2)
     &                     /(gx1*rmx1**2))
                x2_min=atan(((1.d0+gx2**2)*estrinf2**2-rmx2**2)
     &                     /(gx2*rmx2**2))
              ENDIF
*agg
              IF(nit.EQ.2.AND.irepeat.GE.1)THEN
                it2=2
                OPEN(unit=25,file='abvegas_nc.dat',status='old')
                READ(25,*)idum,idum2,iy
                DO i=1,32
                  READ(25,*)iv(i)
                ENDDO !i
                READ(25,*)avgi2,sd2,rchi2a
                DO i=1,50
                  DO j=1,10
                    READ(25,*)y2(i,j)
                  ENDDO !j
                ENDDO !i
                READ(25,*)ndo2,si2,swgt2,schi2
                READ(25,*)rmaxfxn
                CLOSE(25)
                IF(irepeat.EQ.2)THEN
                  nflevts=nflevtstot
                ENDIF
C26
                IF(irepeat.GT.1) nprn=-1
C26
              ENDIF
              IF(ism34.NE.-1) then
                IF(imix.LT.0)THEN
                  ism34=1
                ELSE
                  ism34=0
                ENDIF
              ENDIF
              IF(ism56.NE.-1) then
                IF(imix.LT.0)THEN
                  ism56=1
                ELSE
                  ism56=0
                ENDIF
              ENDIF
              IF(iproc.EQ.7)THEN
                IF(imix.GT.0)THEN
                  rexp5=0.0d0
                  rexp6=0.7d0
                ELSE
                  rexp5=0.0d0
                  rexp6=0.0d0
                ENDIF
              ENDIF
              CALL vegas(region,ndim,fxn,init,ncall,nit,nprn,avgi2,
     &                     sd2,rchi2a,acc,y2,it2,ndo2,si2,swgt2,schi2)
C26
              IF (irepeat.GE.1)  PRINT 205,nevent-nflevts1
C26

              IF(nit.EQ.1)THEN
                rmaxfxn_1it=rmaxfxn
              ENDIF
              IF(nit.EQ.1.AND.istorvegas.EQ.1)THEN
                OPEN(unit=25,file='abvegas_nc.dat',status='new')
                WRITE(25,*)idum,idum2,iy
                DO i=1,32
                  WRITE(25,*)iv(i)
                ENDDO !i
                WRITE(25,*)avgi2,sd2,rchi2a
                DO i=1,50
                  DO j=1,10
                    WRITE(25,*)y2(i,j)
                  ENDDO !j
                ENDDO !i
                WRITE(25,*)ndo2,si2,swgt2,schi2
              ENDIF
              IF(nit.EQ.2.AND.istorvegas.EQ.1)THEN
                rmaxfxn=rmaxfxn_2it
                WRITE(25,*)rmaxfxn
                CLOSE(25)
                WRITE(24,*)rmaxfxn
                WRITE(24,*)avgi1,sd1
                WRITE(24,*)avgi2,sd2
                CLOSE(24)
              ENDIF
*aggend
              resl_nc(it2-1)=resl(it2-1)
              iseed2=idum
              iy2=iy
              idum22=idum2
              DO n=1,32
                iv2(n)=iv(n)
              ENDDO !n
              avgi_tot=avgi1+avgi2
              sd_tot=sqrt(sd1**2+sd2**2)
              IF (irepeat.EQ.2) THEN
                avgi_tot=sigma1+sigma2
                sd_tot=sqrt(sdfin1**2+sdfin2**2)
              ENDIF
            ENDIF
          ENDDO !irip
        ENDDO !nit
      ENDIF
      PRINT*,' '
* em2
C        PRINT *,'ism34 =',ism34,' ism56 =',ism56
C        PRINT *,'rexp5 =',rexp5,' rexp6 =',rexp6
* endem2
      PRINT 151
  151 FORMAT('------------------------------------------------------')
      PRINT*,' '
      PRINT 201,avgi_tot,sd_tot
  201 FORMAT(' Sigma = ',d13.7,' +/-',d10.3,' (pb)')
*agg
      IF(iflat.EQ.1)THEN
        PRINT*,'Informations about flat events generation:'
        PRINT*,'         ----------------------           '
        IF(irepeat.EQ.0)THEN
          PRINT 203,rmaxfxn_1it
  203     FORMAT(' Maximum after first VEGAS iteration = ',d9.3)
        ENDIF
        IF (irepeat.NE.2) THEN
          PRINT 204,rmaxfxn_2it
  204     FORMAT(' Maximum after second VEGAS iteration = ',d9.3)
        ELSE
          PRINT*,'Maximum = ',rmaxfxn_2it
        ENDIF
        PRINT 205,nevent
  205   FORMAT(' Flat events number = ',i9)
        PRINT 206,novermax
  206   FORMAT(' number of function values over maximum = ',i9)
      ENDIF
*aggend
* Distributions:
      IF(idistr.EQ.1)THEN
        OPEN(unit=22,file='ABDIS.DAT',status='new')
        IF(iccnc.EQ.3.OR.(iccnc.EQ.4.AND.icch.EQ.3))THEN
          WRITE(22,*)'N_iteration_CC=',it1-1
          WRITE(22,*)'N_iteration_NC=',it2-1
        ELSE
          WRITE(22,*)'N_iteration=',it1-1
        ENDIF
        WRITE(22,*)'cross-section:',avgi_tot,'+/-',sd_tot,'  (pb)  '
        WRITE(22,*)
        it1=it1-1
        it2=it2-1
        DO m=1,ndistr
          WRITE(22,*)string(m)
          DO n=1,nsubint(m)
            nbintot(m)=nbintot(m)+nbin_number(m,n)
          ENDDO !n
          DO i=1,nbintot(m)
            DO j=1,it1
              IF(ncallbin(m,i,j).ge.2)then
                dev_local(m,i,j)=(ncallbin(m,i,j)*dev_local(m,i,j)-
     &            distr_local(m,i,j)**2)/(ncallbin(m,i,j)-1)
              ENDIF
            ENDDO !j
            IF(iccnc.EQ.3.OR.(iccnc.EQ.4.AND.icch.EQ.3))THEN
              DO j=1,it2
                IF(ncallbin_mix(m,i,j).ge.2)then
                  dev_loc_mix(m,i,j)=(ncallbin_mix(m,i,j)*dev_loc_mix(m,
     &              i,j)-distr_loc_mix(m,i,j)**2)/(ncallbin_mix(m,i,j)
     &              -1)
                ENDIF
              ENDDO !j
            ENDIF
          ENDDO !i
          DO i=1,nbinmax
            devbin(m,i)=0.d0
            IF(iccnc.EQ.3.OR.(iccnc.EQ.4.AND.icch.EQ.3))THEN
              devbin_mix(m,i)=0.d0
            ENDIF
          ENDDO !i
          DO i=1,nbintot(m)
            DO j=1,it1
              IF(dev_local(m,i,j).ne.0.d0)then
                devbin(m,i)=devbin(m,i)+1.d0/dev_local(m,i,j)
              ENDIF
            ENDDO !j
            IF(iccnc.EQ.3.OR.(iccnc.EQ.4.AND.icch.EQ.3))THEN
              DO j=1,it2
                IF(dev_loc_mix(m,i,j).ne.0.d0)then
                  devbin_mix(m,i)=devbin_mix(m,i)+1.d0/
     &                            dev_loc_mix(m,i,j)
                ENDIF
              ENDDO !j
            ENDIF
          ENDDO !i
          DO i=1,nbintot(m)
            DO j=1,it1
              rnorm(m,i,j)=dev_local(m,i,j)*devbin(m,i)
            ENDDO !j
            IF(iccnc.EQ.3.OR.(iccnc.EQ.4.AND.icch.EQ.3))THEN
              DO j=1,it2
                rnorm_mix(m,i,j)=dev_loc_mix(m,i,j)*devbin_mix(m,i)
              ENDDO !j
            ENDIF
          ENDDO !i
          DO i=1,nbintot(m)
            IF(devbin(m,i).gt.0.d0)then
              devbin(m,i)=1.d0/sqrt(devbin(m,i))
            ENDIF
            IF(iccnc.EQ.3.OR.(iccnc.EQ.4.AND.icch.EQ.3))THEN
              IF(devbin_mix(m,i).gt.0.d0)then
                devbin_mix(m,i)=1.d0/sqrt(devbin_mix(m,i))
              ENDIF
            ENDIF
          ENDDO !i
          DO i=1,nbintot(m)
            IF(rnorm(m,i,it1).ne.0.d0)then
              test(m)=test(m)+distr_local(m,i,it1)
              distr_local(m,i,it1)=distr_local(m,i,it1)/rnorm(m,i,it1)
            ENDIF
            IF(iccnc.EQ.3.OR.(iccnc.EQ.4.AND.icch.EQ.3))THEN
              IF(rnorm_mix(m,i,it2).ne.0.d0)then
                test_mix(m)=test_mix(m)+distr_loc_mix(m,i,it2)
                distr_loc_mix(m,i,it2)=distr_loc_mix(m,i,it2)/
     &                                 rnorm_mix(m,i,it2)
              ENDIF
            ENDIF
          ENDDO !i
          DO i=1,nbintot(m)
            DO j=1,it1-1
              IF(rnorm(m,i,j).ne.0.d0)then
                distr_local(m,i,it1)=distr_local(m,i,it1)+
     &            distr_local(m,i,j)/rnorm(m,i,j)
              ENDIF
            ENDDO !j
            IF(iccnc.EQ.3.OR.(iccnc.EQ.4.AND.icch.EQ.3))THEN
              DO j=1,it2-1
                IF(rnorm_mix(m,i,j).ne.0.d0)then
                  distr_loc_mix(m,i,it2)=distr_loc_mix(m,i,it2)+
     &              distr_loc_mix(m,i,j)/rnorm_mix(m,i,j)
                ENDIF
              ENDDO !j
            ENDIF
          ENDDO !i
          IF(iccnc.EQ.3.OR.(iccnc.EQ.4.AND.icch.EQ.3))THEN
            DO i=1,nbintot(m)
              distr_local(m,i,it1)=distr_local(m,i,it1)+
     &                             distr_loc_mix(m,i,it2)
              devbin(m,i)=devbin(m,i)+devbin_mix(m,i)
            ENDDO !i
            tail_local(m,it1)=tail_local(m,it1)+tail_loc_mix(m,it2)
            test(m)=test(m)+test_mix(m)
          ENDIF
          k=0
          DO i=1,nsubint(m)
            delrm=bin_width(m,i)
            rmed=distr_estrinf(m,i)-delrm/2.d0
            IF(i.EQ.1)THEN
              k=1
            ELSE
              k=k+nbin_number(m,i-1)
            ENDIF
            DO j=k,nbin_number(m,i)+k-1
              rmed=rmed+delrm
              WRITE(22,*)rmed,distr_local(m,j,it1)/bin_width(m,i),
     &                   devbin(m,j)/bin_width(m,i)
            ENDDO !j
          ENDDO !i
          WRITE(22,*)'test(m)=',test(m)
          WRITE(22,*)'tail_local(m,it1)=',tail_local(m,it1)
          IF(iccnc.EQ.3.OR.(iccnc.EQ.4.AND.icch.EQ.3))THEN
            WRITE(22,*)'test=',test(m)+tail_local(m,it1)-
     &                         resl_cc(it1)-resl_nc(it2)
          ELSE
            WRITE(22,*)'test=',test(m)+tail_local(m,it1)-resl(it1)
          ENDIF
        ENDDO !m
        CLOSE(22)
      ENDIF

      STOP
      END




      REAL*8 function gammln(xx)

      IMPLICIT NONE
      REAL*8 xx
      INTEGER j
      REAL*8 ser,stp,tmp,xa,y,cof(6)
      SAVE cof,stp
      DATA cof,stp/76.18009172947146d0,-86.50532032941677d0,
     *24.01409824083091d0,-1.231739572450155d0,.1208650973866179d-2,
     *-.5395239384953d-5,2.5066282746310005d0/
      xa=xx
      y=xa
      tmp=xa+5.5d0
      tmp=(xa+0.5d0)*log(tmp)-tmp
      ser=1.000000000190015d0
      DO 11 j=1,6
        y=y+1.d0
        ser=ser+cof(j)/y
   11 CONTINUE
      gammln=tmp+log(stp*ser/xa)
      RETURN
      END
C  (C) Copr. 1986-92 Numerical Recipes Software #>,1')5c).


